SystemOrganization addCategory: #'Phidget-Core'!

!SequenceableCollection methodsFor: '*Phidget-Core' stamp: 'lk 6/16/2009 16:35'!
indexIsInRange: index
	"answer true if the given index is in range of this collection, false otherwise."

	^ index isInteger
		and: [ index >= 1 ]
		and: [ index <= self size ]! !

Object subclass: #PhdCrAttachedDeviceInfo
	instanceVariableNames: 'name serialNumber label deviceClassId deviceTypeId'
	classVariableNames: 'DeviceTypeIdEtoysMap DeviceClassIdMap DeviceTypeIdMap DeviceClassIdCoreMap'
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrAttachedDeviceInfo class methodsFor: 'type and class ID maps' stamp: 'lk 8/6/2009 12:40'!
deviceClassIdCoreMap
	"Answer a map from symbols representing phidget device class IDs in phidget21 to Class objects in PhdCr.
	See the phidget21 documentation at CPhidget_DeviceClass to see where these values come from."

	DeviceClassIdCoreMap ifNotNil: [ ^ DeviceClassIdCoreMap ].
	
	DeviceClassIdCoreMap := Dictionary new: DeviceClassIdMap size.
	DeviceClassIdCoreMap
		add: #Nothing 				-> nil;
		add: #Accelerometer 		-> PhdCrAccelerometer;
		add: #AdvancedServo 		-> PhdCrAdvancedServo;
		add: #Encoder 				-> nil;
		add: #GPS 					-> nil;
		add: #Gyroscope 			-> nil;
		add: #InterfaceKit 			-> PhdCrInterfaceKit;
		add: #LED 					-> nil;	
		add: #MotorControl 			-> PhdCrMotorControl;
		add: #PHSensor 				-> nil;
		add: #RFID 					-> PhdCrRFID;
		add: #Servo 				-> PhdCrServo;
		add: #Stepper 				-> nil;
		add: #TemperatureSensor 	-> nil;
		add: #TextLCD 				-> PhdCrTextLCD;
		add: #TextLED 				-> nil;
		add: #WeightSensor 			-> nil.
	
	^ DeviceClassIdCoreMap! !

!PhdCrAttachedDeviceInfo class methodsFor: 'type and class ID maps' stamp: 'lk 8/6/2009 12:39'!
deviceClassIdMap
	"Answer a map from phidget device class id in phidget21 to symbol representing that class id.
	See the phidget21 documentation at CPhidget_DeviceClass to see where these values come from."

	DeviceClassIdMap ifNotNil: [ ^ DeviceClassIdMap ].
	
	DeviceClassIdMap := Dictionary new: 17.
	DeviceClassIdMap
		add: 1 	-> #Nothing;				"PHIDCLASS_NOTHING"
		add: 2 	-> #Accelerometer;		"PHIDCLASS_ACCELEROMETER"
		add: 3 	-> #AdvancedServo;		"PHIDCLASS_ADVANCEDSERVO"
		add: 4 	-> #Encoder;				"PHIDCLASS_ENCODER"
		add: 5 	-> #GPS;					"PHIDCLASS_GPS"
		add: 6 	-> #Gyroscope;			"PHIDCLASS_GYROSCOPE"
		add: 7 	-> #InterfaceKit;			"PHIDCLASS_INTERFACEKIT"
		add: 8 	-> #LED;					"PHIDCLASS_LED"
		add: 9 	-> #MotorControl;		"PHIDCLASS_MOTORCONTROL"
		add: 10 	-> #PHSensor;			"PHIDCLASS_PHSENSOR"
		add: 11 	-> #RFID;				"PHIDCLASS_RFID"
		add: 12 	-> #Servo;				"PHIDCLASS_SERVO"
		add: 13 	-> #Stepper;				"PHIDCLASS_STEPPER"
		add: 14 	-> #TemperatureSensor;	"PHIDCLASS_TEMPERATURESENSOR"
		add: 15 	-> #TextLCD;				"PHIDCLASS_TEXTLCD"
		add: 16 	-> #TExtLED;				"PHIDCLASS_TEXTLED"
		add: 17 	-> #WeightSensor.		"PHIDCLASS_WEIGHTSENSOR"
	
	^ DeviceClassIdMap! !

!PhdCrAttachedDeviceInfo class methodsFor: 'type and class ID maps' stamp: 'lk 8/6/2009 21:34'!
deviceTypeIdEtoysMap
	"Answer a map from phidget device type id to Class objects in PhdEtoys.
	See the phidget21 documentation at CPhidget_DeviceID to see where these values come from."

	DeviceTypeIdEtoysMap ifNotNil: [ ^ DeviceTypeIdEtoysMap ].
	
	DeviceTypeIdEtoysMap := Dictionary new: DeviceTypeIdMap size.
	DeviceTypeIdEtoysMap
		add: #Accelerometer2Axis 			-> nil;
		add: #Accelerometer3Axis 			-> PhdETAccelerometer;
		add: #AdvancesServo1Motor 			-> nil;
		add: #AdvancesServo8Motor 			-> nil;
		add: #BipolarStepper1Motor 			-> nil;
		add: #Encoder1Encoder1Input 		-> nil;
		add: #EncoderHS1Encoder 			-> nil;
		add: #InterfaceKit004 				-> PhdETInterfaceKit;
		add: #InterfaceKit008 				-> PhdETInterfaceKit;
		add: #InterfaceKit01616 				-> PhdETInterfaceKit;
		add: #InterfaceKit888 				-> PhdETInterfaceKit;
		add: #InterfaceKit888wLCD 			-> PhdETInterfaceKit;
		add: #LED64 						-> nil;
		add: #LinearTouch 					-> nil;
		add: #MotorControlHC2Motor 			-> PhdETMotorControl;
		add: #MotorControlLVMotor4Input 	-> PhdETMotorControl;
		add: #PHSensor 						-> nil;
		add: #RFID2Output 					-> nil;
		add: #RotaryTouch 					-> nil;
		add: #Servo1Motor 					-> PhdETServo1Motor;
		add: #TemperatureSensor 			-> nil;
		add: #TextLCD2x20w888 				-> PhdETTextLCD;
		add: #UnipolarStepper4Motor 		-> nil;
		add: #InterfaceKit088wLCD 			-> PhdETInterfaceKit;
		add: #InterfaceKit488 				-> PhdETInterfaceKit;
		add: #RFID 							-> nil;
		add: #Servo1MotorOld 				-> PhdETServo1Motor;
		add: #Servo4Motor 					-> PhdETServo1Motor;			"not fully supported!!"
		add: #Servo4MotorOld 				-> PhdETServo1Motor;			"not fully supported!!"
		add: #TextLCD2x20 					-> PhdETTextLCD;
		add: #TextLCD2x20w088 				-> PhdETTextLCD;
		add: #TextLED1x8 					-> nil;
		add: #TextLED4x8 					-> nil;
		add: #WeightSensor 					-> nil.

	^ DeviceTypeIdEtoysMap! !

!PhdCrAttachedDeviceInfo class methodsFor: 'type and class ID maps' stamp: 'lk 8/6/2009 12:55'!
deviceTypeIdMap
	"Answer a map from phidget device type id in phidget21 to symbols representing that phidget type.
	See the phidget21 documentation at CPhidget_DeviceID to see where these values come from."

	DeviceTypeIdMap ifNotNil: [ ^ DeviceTypeIdMap ].
	
	DeviceTypeIdMap := Dictionary new: 34.
	DeviceTypeIdMap
		add: 113 	-> #Accelerometer2Axis;					"PHIDID_ACCELEROMETER_2AXIS = 0x071"
		add: 126 	-> #Accelerometer3Axis; 					"PHIDID_ACCELEROMETER_3AXIS = 0x07E"
		add: 130 	-> #AdvancedServo1Motor;				"PHIDID_ADVANCEDSERVO_1MOTOR = 0x082"
		add: 58 		-> #AdvancedServo8Motor;				"PHIDID_ADVANCEDSERVO_8MOTOR = 0x03A"
		add: 123 	-> #BipolarStepper1Motor;				"PHIDID_BIPOLAR_STEPPER_1MOTOR = 0x07B"
		add: 75 		-> #Encoder1Encoder1Input;				"PHIDID_ENCODER_1ENCODER_1INPUT = 0x04B"
		add: 128 	-> #EncoderHS1Encoder;					"PHIDID_ENCODER_HS_1ENCODER = 0x080"
		add: 64 		-> #InterfaceKit004;						"PHIDID_INTERFACEKIT_0_0_4 = 0x040"
		add: 129 	-> #InterfaceKit008;						"PHIDID_INTERFACEKIT_0_0_8 = 0x081"
		add: 68 		-> #InterfaceKit01616;					"PHIDID_INTERFACEKIT_0_16_16 = 0x044"
		add: 69 		-> #InterfaceKit888;						"PHIDID_INTERFACEKIT_8_8_8 = 0x045"
		add: 125 	-> #InterfaceKit888wLCD;					"PHIDID_INTERFACEKIT_8_8_8_w_LCD = 0x07D"
		add: 74 		-> #LED64;								"PHIDID_LED_64 = 0x04A"
		add: 118 	-> #LinearTouch;						"PHIDID_LINEAR_TOUCH = 0x076"
		add: 89 		-> #MotorControlHC2Motor;				"PHIDID_MOTORCONTROL_HC_2MOTOR = 0x059"
		add: 88 		-> #MotorControlLV2Motor4Input;			"PHIDID_MOTORCONTROL_LV_2MOTOR_4INPUT = 0x058"
		add: 116 	-> #PHSensor;							"PHIDID_PHSENSOR = 0x074"
		add: 49 		-> #RFID2Output;							"PHIDID_RFID_2OUTPUT = 0x031"
		add: 119 	-> #RotaryTouch;						"PHIDID_ROTARY_TOUCH = 0x077"
		add: 57 		-> #Servo1Motor;						"PHIDID_SERVO_1MOTOR = 0x039"
		add: 112 	-> #TemperatureSensor;					"PHIDID_TEMPERATURESENSOR = 0x070"
		add: 381 	-> #TextLCD2x20w888;					"PHIDID_TEXTLCD_2x20_w_8_8_8 = 0x17D"
		add: 122 	-> #UnipolarStepper4Motor;				"PHIDID_UNIPOLAR_STEPPER_4MOTOR = 0x07A"
		add: 83 		-> #InterfaceKit088wLCD;					"PHIDID_INTERFACEKIT_0_8_8_w_LCD = 0x053"
		add: 4 		-> #InterfaceKit488;						"PHIDID_INTERFACEKIT_4_8_8 = 4"
		add: 48 		-> #RFID;								"PHIDID_RFID = 0x030"
		add: 2 		-> #Servo1MotorOld;						"PHIDID_SERVO_1MOTOR_OLD = 2"
		add: 56 		-> #Servo4Motor;						"PHIDID_SERVO_4MOTOR = 0x038"
		add: 3 		-> #Servo4MotorOld;						"PHIDID_SERVO_4MOTOR_OLD = 3"
		add: 82 		-> #TextLCD2x20;							"PHIDID_TEXTLCD_2x20 = 0x052"
		add: 339 	-> #TextLCD2x20w088;					"PHIDID_TEXTLCD_2x20_w_0_8_8 = 0x153"
		add: 73 		-> #TextLED1x8;							"PHIDID_TEXTLED_1x8 = 0x049"
		add: 72 		-> #TextLED4x8;							"PHIDID_TEXTLED_4x8 = 0x048"
		add: 114 	-> #WeightSensor.						"PHIDID_WEIGHTSENSOR = 0x072"		

	^ DeviceTypeIdMap! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/11/2009 16:34'!
asStringArray

	^ {	self deviceName.
		self serialNumber asString.
		self label.
		self deviceClassId asString.
		self deviceTypeId asString	}! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 11:18'!
deviceClassId
	^ deviceClassId! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 11:17'!
deviceClassId: classId
	deviceClassId := classId! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 10:16'!
deviceName
	^ name! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 10:16'!
deviceName: aString
	name := aString! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 11:18'!
deviceTypeId
	^ deviceTypeId! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 11:18'!
deviceTypeId: typeId
	deviceTypeId := typeId! !

!PhdCrAttachedDeviceInfo methodsFor: 'create Phidget objects' stamp: 'lk 8/6/2009 15:53'!
getCoreObject
	"Answer a opened PhdCr object representing myself.
	If an instance cannot be created, answer nil."
	
	| classId classObject object |
	
	"get class"
	classId := self class deviceClassIdMap
		at: self deviceClassId
		ifAbsent: [ nil ].
	classId ifNil: [ ^ nil ].
	classObject := self class deviceClassIdCoreMap
		at: classId
		ifAbsent: [ nil ].
	classObject ifNil: [ ^ nil ].
	
	"instantiate the class with my serial number"
	object := classObject new openWithSerial: self serialNumber.
	object isOpen ifFalse: [ object close. ^ nil ].
	
	^ object! !

!PhdCrAttachedDeviceInfo methodsFor: 'create Phidget objects' stamp: 'lk 8/6/2009 15:53'!
getEtoysObject
	"Answer a opened PhdET object representing myself.
	If an instance cannot be created, answer nil."
	
	| typeId classObject |
	
	"get class"
	typeId := self class deviceTypeIdMap
		at: self deviceTypeId
		ifAbsent: [ nil ].
	typeId ifNil: [ ^ nil ].
	classObject := self class deviceTypeIdEtoysMap
		at: typeId
		ifAbsent: [ nil ].
	classObject ifNil: [ ^ nil ].
	
	"instantiate the class"
	^ classObject new.! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 10:16'!
label
	^ label! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 10:16'!
label: aString
	label := aString! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 10:16'!
serialNumber
	^ serialNumber! !

!PhdCrAttachedDeviceInfo methodsFor: 'accessing' stamp: 'lk 8/6/2009 10:17'!
serialNumber: anInteger
	serialNumber := anInteger! !

Object subclass: #PhdCrComponent
	instanceVariableNames: 'phidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrComponent commentStamp: 'lk 6/16/2009 16:45' prior: 0!
I am an abstract class for any Components a Phidget can have.


Instance Variables
	phidget:		<PhdCrPhidget>

phidget
	- the 'parent' of this component. Every component belongs to a Phidget. Only if the Phidget is attached the component is usable.
	- I need this value to invoke the primitives to talk with the plugin/Phidget hardware, because every primitive I know needs the Phidget's unique identifier (id).


I must be created with an instance of PhdCrPhidget as parameter that sets my 'phidget' instance variable (see 'class newWith:' and 'setParent').!

PhdCrComponent subclass: #PhdCrCompRFID
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompRFID commentStamp: 'lk 6/16/2009 19:45' prior: 0!
I am a special Phidget Component for reading RFID tags.
I have an antenna that can be enabled or disabled (getter/setter exist).
When my antenna is on I can read information about a near tag. Actually I only know which was the last tag I read, and whether I am currently reading any. The Phidget I belong to can signal when I found or lost a tag. (getter)

Currently I only belong to PhdCrRFID.
Example for usage there:
	rfid := PhdCrRFID new.
	rfid
		"The antenna might still be disabled, so enable it as soon as the Phidget is attached."
		on: #Attach do: [ rfid rfidSensor setAntennaState: true ];
		"Print to Transcript whenever we find or lose a tag. Print its identifier."
		on: #TagFound do: [ Transcript show: 'Tag found: ', rfid rfidSensor currentTag; cr ];
		on: #TagLost do: [ Transcript show: 'Tag lost: ', rfid rfidSensor lastTag; cr ];
		openAny.
	"Now attach a RFID Phidget, hold a tag near the Phidget and then take it away."
	rfid close.
!

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primGetLastTag: phidgetIdArray
	"Answer a array of 5 bytes representing the id of the tag last read by this phidget, or nil if there was an error."

	<primitive: 'primGetLastRFIDTag' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primHasTag: phidgetIdArray
	"Answer 1 if the Phidget currently reads a tag, or other values otherwise."
	
	<primitive: 'primHasRFIDTag' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primIsAntennaOn: phidgetIdArray
	"Answer 1 if the antenna is on, or other values otherwise."
	
	<primitive: 'primIsRFIDAntennaOn' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetAntennaState: phidgetIdArray to: state
	"Set the state of the antenna to on (state=1) or off (state=0).
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetRFIDAntennaState' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetSemaphoreIndexTagFound: semaphoreIndex for: phidgetIdArray
	"registers the given semaphore to be signaled when a tag is found by the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexRFIDTagFound' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetSemaphoreIndexTagLost: semaphoreIndex for: phidgetIdArray
	"registers the given semaphore to be signaled when a tag is lost by the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexRFIDTagLost' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:37'!
currentTag
	"Returns a string which represents a hex-value of the 5 bytes tag ID of the tag currently read.
	If there is no tag currently read, return an empty string."
	
	^ self hasTag
		ifTrue: [ self lastTag ]
		ifFalse: [ '' ]! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 17:19'!
currentTagBytes
	"Returns a byte array with the 5 elements of the tag ID currently read.
	If there is no tag currently read, return nil."
	
	^ self hasTag
		ifTrue: [ self lastTagBytes ]
		ifFalse: [ nil ]! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:34'!
hasTag
	"Returns true is a tag is currently read and false if not."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primHasTag: phidget id) == 1! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'pt 7/6/2009 17:00'!
isAntennaOn
	"Answer true if the antenna is on, false otherwise."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primIsAntennaOn: phidget id) == 1! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'pt 7/6/2009 17:00'!
isAntennaOn: aBoolean 
	"Sets the state of the antenna to true or false.
	The antenna must be enabled in order to find tags near the Phidget.
	Returns true if setting the antenna state was successful, false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class 
		primSetAntennaState: phidget id
		to: anInt) == 1! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:38'!
lastTag
	"Returns a string which represents a hex-value of the 5 bytes tag ID last read by the sensor.
	This tag may or may not still be on the reader.
	If there was no tag read by the reader since attached, the method returns a string with 10 zeros.
	If the Phidget with this component is not attached return an empty string."
	
	| aString bytes |
	aString := ''.
	bytes := self lastTagBytes.
	bytes ifNotNil: [
		bytes do: [ :byte | aString := aString , byte hex ] ].
	^ aString! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:39'!
lastTagBytes
	"Returns a byte array with the 5 elements of the tag ID that was last read by the sensor.
	This tag may or may not still be on the reader.
	If there was no tag read by the reader since opened, the method returns a Byte array with zeros.
	If the Phidget is not attached return nil."
	
	phidget isAttached ifFalse: [ ^ nil ].
	^ self class primGetLastTag: phidget id! !

PhdCrComponent subclass: #PhdCrCompSimpleLED
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompSimpleLED commentStamp: 'lk 6/16/2009 19:46' prior: 0!
I am a special Phidget Component representing a simple LED.
I can be turned off or on (getter/setter exist).

Currently I only belong to PhdCrRFID.!

!PhdCrCompSimpleLED class methodsFor: 'primitives' stamp: 'lk 7/6/2009 09:57'!
primIsOn: phidgetIdArray
	"Answer 1 if theLED is on, or other values otherwise."
	
	<primitive: 'primIsSimpleLEDOn' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompSimpleLED class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetState: phidgetIdArray to: state
	"Set the state of the LED to true (state=1) or false (state=0).
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSimpleLEDState' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompSimpleLED methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:41'!
isOn
	"Answer true if the phidget is attached and the LED is on, else false."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primIsOn: phidget id) > 0! !

!PhdCrCompSimpleLED methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:13'!
isOn: aBoolean 
	"Sets the state to true or false.
	true meens the LED is illuminated, false that it is not.
	returns true if setting was successful, and false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetState: phidget id
		to: anInt) == 1! !

PhdCrComponent subclass: #PhdCrCompTextDisplay
	instanceVariableNames: 'text displayedText scrollPosition running seperator waitTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompTextDisplay commentStamp: 'pt 7/20/2009 16:26' prior: 0!
I am an abstract Phidget Component that can display texts on a display.

I have a number of rows and columns. In one cell I can display one character.
You can ask me to display a character at one cell (displayCharacter:inRow:andColumn:), or to display a string in a row (displayString:inRow:). If the string is too long it will be truncated to fit the line.
The original - too long - text is cached, so that is can be displayed if you enable scrolling.
You can ask me to scroll through the text in an endless loop (startScrolling:, stopScrolling:), also for each row seperately.
Scrolling speed is controlled with my waitTime. My seperator is displayed between start and end of the string.

Note:
 - When the phidget is attached it should call 'initializeRows', when it is detached 'initializeRowsEmpty'. Else there will be displaying errors.
 - Each phidget that uses me should implement #textChanged

Instance Variables
	running:		Dictionary (containing Booleans)
	scrollPosition:		Dictionary (containing Integers)
	seperator:		String
	text:		Dictionary (containing Strings)
	waitTime:		Integer

running
	- This is a Dictionary of the form {row -> Boolean} where row is the number of a row. The Boolean indicates wether for that row scolling is enabled or not.

scrollPosition
	- This is a Dictionary of the form {row -> position} where row is the number of a row. The position is used if scrolling is enabled for a row. It indicates the character of the text that has to be displayed in the first column.
	
seperator
	- The seperator is a String seperating the end and the beginning of a row if scrolling is enabled.
	  If you make the string 'I love ice cream' a scrolling text on the display. It will be displayed as 'I love ice cream ** I love ice cream ** I love ice cream ** I love[...]' if the seperator is ' ** '.

text
	- This is a Dictionary of the form {row -> String} where row is the number of a row. The String is the cached text I try to display. If the text is too long I display it truncated (but chache the original value here).

waitTime
	-This Integer indicates the number of milliseconds to wait between a scroll step and another.
!

!PhdCrCompTextDisplay class methodsFor: 'primitives' stamp: 'lk 7/6/2009 09:56'!
primColumnCount: phidgetIdArray
	"Answer the number of columns this display has."
	
	<primitive: 'primGetTextDisplayColumnCount' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompTextDisplay class methodsFor: 'primitives' stamp: 'lk 7/6/2009 10:10'!
primDisplayCharacter: phidgetIdArray in: row and: column value: charStringArray
	"Set the charachter in row 'row' and column 'coulumn' to charStringArray.
	charStringArray is _not_ a Character, but a String exact length of 1."
	
	<primitive: 'primSetTextDisplayCharacter' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompTextDisplay class methodsFor: 'primitives' stamp: 'lk 7/6/2009 10:34'!
primDisplayString: phidgetIdArray in: row value: stringArray
	"Set the text in row 'row' to stringArray. The string must be <= row count."
	
	<primitive: 'primSetTextDisplayString' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompTextDisplay class methodsFor: 'primitives' stamp: 'lk 7/6/2009 09:55'!
primRowCount: phidgetIdArray
	"Answer the number of rows this display has."
	
	<primitive: 'primGetTextDisplayRowCount' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompTextDisplay methodsFor: 'display' stamp: 'lk 7/6/2009 10:58'!
clear

	1 to: self rows do: [ :rowNumber | self clearRow: rowNumber ].! !

!PhdCrCompTextDisplay methodsFor: 'display' stamp: 'lk 7/6/2009 10:46'!
clearRow: anInteger

	self displayString: '' inRow: anInteger.! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'lk 7/6/2009 16:02'!
columnIsInRange: anInteger

	phidget isAttached ifFalse: [ ^ false ].
	^ anInteger > 0 & (anInteger <= self columns)! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'lk 7/6/2009 10:07'!
columns
	"Answer the number of columns of this display."
	
	phidget isAttached ifFalse: [ ^ 0 ].
	^ (self class primColumnCount: phidget id)! !

!PhdCrCompTextDisplay methodsFor: 'display' stamp: 'lk 7/9/2009 14:06'!
displayCharacter: aCharacterOrString inRow: row andColumn: column
	"display character or 1-element string in row 'row' and column 'column'.
	return true if the operation succeeded, false otherwise."
	
	| charString |
	phidget isAttached ifFalse: [ ^ false ].
	(self rowIsInRange: row) & (self columnIsInRange: column) ifFalse: [ ^ false ].
	
	charString := nil.
	(aCharacterOrString isMemberOf: Character)
		ifTrue: [ charString := aCharacterOrString asString ]
		ifFalse: [
			(aCharacterOrString isMemberOf: ByteString)
				ifTrue: [ charString := aCharacterOrString ]
				ifFalse: [ ^ false ] ].
	charString size == 1 ifFalse: [ ^ false ].
	
	^ (self class
		primDisplayCharacter: phidget id
		in: row - 1
		and: column - 1
		value: charString) == 1! !

!PhdCrCompTextDisplay methodsFor: 'display' stamp: 'lk 7/8/2009 18:44'!
displayRow: aNumber

	(self rowIsInRange: aNumber) ifFalse: [ ^ self ].
	self displayString: (text at: aNumber) inRow: aNumber.! !

!PhdCrCompTextDisplay methodsFor: 'display' stamp: 'pt 7/9/2009 12:09'!
displayString: aString inRow: anInteger
	"displays aString in row anInteger.
	if the string is longer than self rows, it is truncated to a valid size.
	return true if the operation succeeded, false otherwise.
	Chaches the string."

	phidget isAttached ifFalse: [ ^ false ].
	(self rowIsInRange: anInteger) ifFalse: [ ^ false ].
	
	text at: anInteger put: aString.
	^ self displayStringWithoutCaching: aString inRow: anInteger! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'pt 7/14/2009 16:48'!
displayStringWithoutCaching: aString inRow: anInteger
	"display string aString in row anInteger.
	if the string is longer than self rows, it is truncated to a valid size.
	return true if the operation succeeded, false otherwise."

	| displayString result |
	phidget isAttached ifFalse: [ ^ false ].
	(self rowIsInRange: anInteger) ifFalse: [ ^ false ].
	displayString := self getValidStringFrom: aString.
	displayString ifNil: [ ^ false ].
	
	result := (self class
		primDisplayString: phidget id
		in: anInteger - 1
		value: displayString) == 1.
	displayedText at: anInteger put: displayString.
	self textChanged.
	^ result! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'pt 7/14/2009 16:50'!
displayedTextAt: anInteger
	"Returns the actiually displayed text in the given row."
	
	(displayedText indexIsInRange: anInteger) ifFalse: [ ^ '' ].
	^ displayedText at: anInteger! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/9/2009 11:40'!
expandString: aString toMinLength: length
	"Expands the given string through concating it with itself until its length is exact or greater than the given length.
	 Assumes that the given string is not empty!!"
	
	| expandedText |
	self assert: [ aString size > 0 ].
	
	expandedText := ''.
	[ expandedText size < length ] whileTrue: [
		expandedText := expandedText , aString asString ].	
		
	^ expandedText! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'lk 7/6/2009 10:44'!
getValidStringFrom: aString

	phidget isAttached ifFalse: [ ^ nil ].
	(aString isKindOf: ByteString) ifFalse: [ ^ nil ].
	^ aString truncateTo: self columns.! !

!PhdCrCompTextDisplay methodsFor: 'initialisation' stamp: 'pt 7/9/2009 13:43'!
initialize

	super initialize.
	
	self
		initializeRowsEmpty; "I don't have rows until I am attached."
		resetScrollPosition.
	seperator := ' ** '.
	waitTime := 300.! !

!PhdCrCompTextDisplay methodsFor: 'initialisation' stamp: 'pt 7/14/2009 16:46'!
initializeRows

	| rows |
	rows := self rows.
	
	text := (Array new: rows).
	displayedText := (Array new: rows).
	running := (Array new: rows).
	1 to: rows do: [ :rowNumber |
		text at: rowNumber put: ''.
		displayedText at: rowNumber put: ''.
		running at: rowNumber put: false ].! !

!PhdCrCompTextDisplay methodsFor: 'initialisation' stamp: 'pt 7/14/2009 16:46'!
initializeRowsEmpty
	
	text := Array new.
	displayedText := Array new.
	running := Array new.! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/14/2009 16:00'!
isRowScrolling: aNumber

	(self rowIsInRange: aNumber) ifFalse: [ ^ false ].
	
	^ running at: aNumber.! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/9/2009 11:32'!
pasteTextFromRow: aNumber
	"A Method used to support scrolling.
	Combines the text of a specific row and the seperator. Ensures that the pasteText never is an empty String."
	
	| pasteText |
	pasteText := (text at: aNumber) asString , seperator asString.
	(pasteText == '') ifTrue: [ ^ ' ' ].
	^ pasteText! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/9/2009 11:53'!
resetScrollPosition
	"Resets the scrollPosition so that the text to Display looks like there is no scrolling."
	
	scrollPosition := 1.! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'lk 7/6/2009 10:30'!
rowIsInRange: anInteger

	phidget isAttached ifFalse: [ ^ false ].
	^ anInteger > 0 & (anInteger <= self rows)! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'lk 7/6/2009 10:06'!
rows
	"Answer the number of rows of this display."
	
	phidget isAttached ifFalse: [ ^ 0 ].
	^ (self class primRowCount: phidget id)! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/14/2009 16:29'!
scrollRowProcess: aNumber 
	"This is the main method running in a process forked in startScrollingRow:.
	It's intend is to let the text in the given row scroll.'"
	| displayString expandedText pasteText |
	
	"displayString - the final output string that I'll give the device to display
	 expandedText - a string where the text and the seperator will be copied into until it is large enough to extract the displayString from it.
	 pasteText      - a (never empty) string, that contains the text and the seperator."
	
	[ (self rowIsInRange: aNumber) and: [running at: aNumber] ] whileTrue: [
		"If the given row is not in the range of the given device, stop here."
		
		"calculates the text to display and display it"
		pasteText := self pasteTextFromRow: aNumber.
		expandedText := self expandString: pasteText toMinLength: scrollPosition + self columns.
		displayString := self getValidStringFrom: (expandedText copyFrom: scrollPosition to: expandedText size).
		self displayStringWithoutCaching: displayString	inRow: aNumber.
		"calculate the new scrollPosition"
		scrollPosition := scrollPosition + 1.
		(scrollPosition > ((text at: aNumber) size + seperator size)) ifTrue: [ self resetScrollPosition ].
		"wait for <waitTime> milliseconds so that the scroll doesn't look too fast"
		(Delay forMilliseconds: self waitTime) wait ].! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'lk 7/8/2009 18:48'!
seperator

	^ seperator! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/9/2009 11:57'!
seperator: aString

	self resetScrollPosition.
	seperator := aString asString.! !

!PhdCrCompTextDisplay methodsFor: 'display' stamp: 'pt 7/9/2009 11:56'!
setText: aString inRow: aNumber

	(self rowIsInRange: aNumber) ifFalse: [ ^ self ].
	
	self resetScrollPosition.
	text at: aNumber put: aString.! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/8/2009 17:46'!
startScrolling

	(1 to: self rows) do: [:i |
		self startScrollingRow: i].! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'lk 7/8/2009 18:47'!
startScrollingRow: aNumber

	(self rowIsInRange: aNumber) ifFalse: [ ^ self ].
	
	running at: aNumber put: true.
	[ self scrollRowProcess: aNumber ] fork.! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/8/2009 17:47'!
stopScrolling

	(1 to: self rows) do: [:i |
		self stopScrollingRow: i].! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'pt 7/15/2009 09:01'!
stopScrollingRow: aNumber

	(self rowIsInRange: aNumber) ifFalse: [ ^ self ].
	running at: aNumber put: false.
	self displayRow: aNumber.! !

!PhdCrCompTextDisplay methodsFor: 'private' stamp: 'pt 7/14/2009 16:55'!
textChanged
	"Tell my phidget that my text has changed."
	
	phidget textChanged.! !

!PhdCrCompTextDisplay methodsFor: 'display' stamp: 'lk 7/8/2009 18:50'!
textInRow: aNumber

	(self rowIsInRange: aNumber) ifFalse: [ ^ nil ].
	^ text at: aNumber! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'fs 7/8/2009 16:49'!
waitTime

	^ waitTime! !

!PhdCrCompTextDisplay methodsFor: 'scrolling' stamp: 'fs 7/8/2009 16:49'!
waitTime: aNumber

	waitTime := aNumber.! !

PhdCrCompTextDisplay subclass: #PhdCrCompTextLCD
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompTextLCD commentStamp: 'lk 7/9/2009 14:35' prior: 0!
I am a special Phidget Component that can display texts on a LCD display.

There are different values that can be retrieved and set:
	- contrast: integer between 0 and 255 representing the current display contrast.
	- isBacklightOn: boolean. If it is true, the display's backlight is enabled.
	- isCursorOn: a boolean. If it is true, a cursor is shown after the last written character.
	- isCursorBlinking: a bollean. If it is true, the cursor is blinking.

I currently only belong to PhdCrTextLCD.!

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:21'!
primBacklightIsOn: phidgetIdArray
	"Answer 1 if the backlight is on, or other values otherwise."
	
	<primitive: 'primIsTextLCDBacklightOn' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:37'!
primCursorIsBlinking: phidgetIdArray
	"Answer 1 if the cursor is blinking, or other values otherwise."
	
	<primitive: 'primIsTextLCDCursorBlinking' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:28'!
primCursorIsOn: phidgetIdArray
	"Answer 1 if the cursor is on, or other values otherwise."
	
	<primitive: 'primIsTextLCDCursorOn' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:40'!
primGetContrast: phidgetHandle
	"Gets the contrast (0..255) of the given phidget."

	<primitive: 'primGetTextLCDContrast' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:27'!
primSetBacklightOn: phidgetIdArray to: state
	"Set the state of the backlight to enabled (state=1) or disabled (state=0).
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetTextLCDBacklightOn' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:42'!
primSetContrast: phidgetHandle to: aNumber
	"Sets the contrast of the given phidget to aNumber (0..255).
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetTextLCDContrast' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:36'!
primSetCursorBlinking: phidgetIdArray to: state
	"Set the state of the cursor to blinking (state=1) or not blinking (state=0).
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetTextLCDCursorBlinking' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompTextLCD class methodsFor: 'primitives' stamp: 'lk 7/6/2009 15:29'!
primSetCursorOn: phidgetIdArray to: state
	"Set the state of the cursor to enabled (state=1) or disabled (state=0).
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetTextLCDCursorOn' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:51'!
contrast
	"Gets the contrast of the display (0..255).
	if there was an error, return 0."
	
	phidget isAttached ifFalse: [ ^ 0 ].
	^ self class primGetContrast: phidget id! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:55'!
contrast: anInteger
	"Sets the contrast of the display to aNumber (0..255).
	Return true if the operation suceeded, false otherwise."
	
	phidget isAttached ifFalse: [ ^ false ].
	anInteger < 0 | anInteger > 255 ifTrue: [ ^ false ].
	^ (self class
		primSetContrast: phidget id
		to: anInteger) == 1! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:20'!
isBacklightOn
	"Answer true if the phidget is attached and the backlight is on, else false."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primBacklightIsOn: phidget id) > 0! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:48'!
isCursorBlinking
	"Answer true if the phidget is attached and the cursor is blinking, else false."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primCursorIsBlinking: phidget id) > 0! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:46'!
isCursorOn
	"Answer true if the phidget is attached and the cursor is on, else false."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primCursorIsOn: phidget id) > 0! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:44'!
setBacklightOn: aBoolean 
	"Sets the backlight state to true (enabled) or false (disabled).
	returns true if setting was successful, and false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetBacklightOn: phidget id
		to: anInt) == 1! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:47'!
setCursorBlinking: aBoolean 
	"Sets the cursor state to blinking (aBoolean=true) or not blinking (aBoolean=false).
	returns true if setting was successful, and false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetCursorBlinking: phidget id
		to: anInt) == 1! !

!PhdCrCompTextLCD methodsFor: 'accessing' stamp: 'lk 7/6/2009 15:46'!
setCursorOn: aBoolean 
	"Sets the cursor state to true (enabled) or false (disabled).
	returns true if setting was successful, and false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetCursorOn: phidget id
		to: anInt) == 1! !

!PhdCrComponent class methodsFor: 'instance creation' stamp: 'lk 6/16/2009 20:19'!
newWith: phidget
	"create a new Component with given phidget. This Phidget, once set, will never change again."

	^ self new
		setParent: phidget! !

!PhdCrComponent methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:35'!
on: aSymbol do: aBlock
	"When event aSymbol is signaled for my Phidget, do aBlock.
	This method is provided to have a moren natural interface.
	See PhdCrPhidget on:do: for more information."
	
	phidget on: aSymbol do: aBlock! !

!PhdCrComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 17:21'!
setParent: parent
	"Set my 'phidget' instance variable to the given one.
	If I already have a parent, do nothing."
	
	phidget ifNil: [ phidget := parent ].! !

PhdCrComponent subclass: #PhdCrMultiComponent
	instanceVariableNames: 'index nextLink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrMultiComponent commentStamp: 'lk 6/16/2009 18:56' prior: 0!
I am an abstract Phidget Component for components that cann exist serveral times on a Phidget, but the exact number differs from Phidget to Phidget. (The Phidget type allows different numbers).


Instance Variables
	index:		<Integer>
	nextLink:	<nil or PhdCrMultiComponent>

index
	- number that identifies me on a Phidgets that has more instances of my class. Like 'parent' it is provided on construction and will never change again.
	- Needed on every primitve invocation, except for the primitive getting the number of equal components (componentCount).

nextLink
	- Needed to be member of a linked list. Can be ignored for Phidget understanding.


About MultiComponents:
	When a Phidget with a multi-component is attached, it initializes the list containing instances of this class. It uses my classes 'getListFor:' method to do this.
	'getListFor:' dynamically gets the number of similar components and adds one list item for each to the list. It calls 'primComponentCount:' to determine this number. This method must be overritten in subclasses to point to the right primitive.
	A MultiComponent is created by sending the 'newWith:index:' method to my class, because like in simple Components it needs a phidget 'parent', and for Multicomponents it aditionally needs its index (see above).
!

PhdCrMultiComponent subclass: #PhdCrCompAnalogSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompAnalogSensor commentStamp: 'sp 7/1/2009 14:39' prior: 0!
I am a special Phidget Component for analog sensor values.

I can be asked for a value I am currently reading, from 0 to 1000 (getter exists).

My Phidget can call a user defined callback block when my value changes, when prior
	analogSensor on: #SensorChange do: [ "user defined content" ]
was called. Note: This block will only be evaluated if the change of the value is equal or bigger than my changeTrigger.
My changeTrigger is a value between 0 and 100. (getter/setter exist).

Currently I only belong to PhdCrInterfaceKit.!

!PhdCrCompAnalogSensor class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:55'!
primComponentCount: phidgetIdArray
	"Gets the number of sensors for a phidget analog sensors component."

	<primitive: 'primNumAnalogSensors' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 7/20/2009 15:55'!
primGetChangeTrigger: phidgetIdArray at: index
	"Gets the current change trigger of the given Phidget's sensor at given index."

	<primitive: 'primGetIntegerChangeTrigger' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:55'!
primGetValue: phidgetIdArray at: index
	"Gets the sensor value (0-1000) at the analog input with the given index."

	<primitive: 'primGetAnalogSensorValue' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 7/20/2009 15:56'!
primSetChangeTrigger: phidgetIdArray at: index to: anInteger
	"Sets the change trigger of given Phidget at given sensor index to the given value.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetIntegerChangeTrigger' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:55'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an sensor value changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAnalogSensorChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAnalogSensor methodsFor: 'accessing' stamp: 'lk 7/20/2009 15:57'!
changeTrigger

	phidget isAttached ifFalse: [ ^ 0 ].
	^ self class
		primGetChangeTrigger: phidget id
		at: index - 1! !

!PhdCrCompAnalogSensor methodsFor: 'accessing' stamp: 'lk 7/20/2009 15:57'!
changeTrigger: anInteger
	"Sets the sensor change trigger of the sensor to 'anInteger'.
	a sensor change block will only be evaluated if the change is more than this trigger."
	
	phidget isAttached ifFalse: [ ^ false ].
	(anInteger isInRange: (0 to: 100)) ifFalse: [ ^ false ].

	^ (self class
		primSetChangeTrigger: phidget id
		at: index - 1
		to: anInteger) == 1! !

!PhdCrCompAnalogSensor methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:53'!
value
	"Gets the current value of the sensor (integer from 0 to 1000)."
	
	phidget isAttached ifFalse: [ ^ 0 ].
	^ self class
		primGetValue: phidget id
		at: index - 1! !

PhdCrMultiComponent subclass: #PhdCrCompAxis
	instanceVariableNames: 'sensitivity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompAxis commentStamp: 'sp 7/1/2009 14:38' prior: 0!
I am a special Phidget Component for axes.

I can be asked for the current acceleration of an axis of me, which can be retrieved and set (getter/setter exist). This acceleration should be between my minimal and maximal position which can also be retrieved. Be careful that if those values cannot be retrieved (for instance if the Phidget is not attached) they return a value of 0.0, although this may also be a valid value.

My Phidget can call a user defined callback block when my acceleration changes, when prior
	axis on: #AccelerationChange do: [ "user defined content" ]
was called. Note: This block will only be evaluated if the change of the value is equal or bigger than my changeTrigger.
My changeTrigger is a value between 0 and 1 (getter/setter exist).
By default it is set to 0.05, so that there will be no acceleration when I am not in move.

Currently I only belong to PhdCrAccelerometer.!

!PhdCrCompAxis class methodsFor: 'multi-component' stamp: 'sp 6/18/2009 21:59'!
primComponentCount: phidgetHandle
	"Gets the number of axes for a phidget axis component."

	<primitive: 'primNumAxes' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'sp 6/24/2009 14:35'!
primGetAcceleration: phidgetHandle at: index
	"Gets the acceleration of the given axis."

	<primitive: 'primGetAxisAcceleration' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'lk 7/20/2009 16:00'!
primGetChangeTrigger: phidgetHandle at: index

	<primitive: 'primGetIndexedDoubleChangeTrigger' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'sp 6/21/2009 10:15'!
primGetMaxAcceleration: phidgetIdArray at: index
	"Returns a float containing max. possible acceleration of the axis at the given index."

	<primitive: 'primGetAxisAccelerationMax' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'sp 6/21/2009 10:15'!
primGetMinAcceleration: phidgetIdArray at: index
	"Returns a float containing min. possible acceleration of the axis at the given index."

	<primitive: 'primGetAxisAccelerationMin' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'lk 7/20/2009 16:01'!
primSetChangeTrigger: phidgetHandle at: index to: anInteger

	<primitive: 'primSetIndexedDoubleChangeTrigger' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'sp 6/18/2009 21:30'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an acceleration occurs on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAxisChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/26/2009 11:12'!
acceleration
	"Gets the current raw acceleration of the axis (double between min and max), multiplied by self sensitivity."
	
	^ self accelerationRaw * self sensitivity! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'sp 6/24/2009 14:32'!
accelerationMax
	"Gets the max. acceleration of the axis.
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAccelerationMax: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'sp 6/24/2009 14:32'!
accelerationMin
	"Gets the min. acceleration of the axis.
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAccelerationMin: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/26/2009 11:11'!
accelerationRaw
	"Gets the current acceleration of the axis (double between min and max).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAcceleration: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'prototyping' stamp: 'pt 6/23/2009 18:44'!
angleDegrees

	^ self angleRadians radiansToDegrees! !

!PhdCrCompAxis methodsFor: 'prototyping' stamp: 'pt 6/23/2009 19:08'!
angleDegreesNumber 

	^ self angleDegrees + 90.0 / 180.0 * 1000.0! !

!PhdCrCompAxis methodsFor: 'prototyping' stamp: 'lk 6/26/2009 11:10'!
angleRadians

	| normalized |
	normalized := self acceleration.
	normalized < -1.0 ifTrue: [ normalized := -1.0 ].
	normalized > 1.0 ifTrue: [ normalized := 1.0 ].
	
	^ normalized arcSin! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 7/20/2009 15:57'!
changeTrigger
	"Gets the current change trigger of the axis (double between 0.0 and 1.0).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetChangeTrigger: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 7/20/2009 15:57'!
changeTrigger: aDouble
	"Sets the acceleration change trigger of the axis to 'aDouble'.
	a acceleration change block will only be evaluated if the change is more than this trigger.
	Return true if the operation succeeds, false otherwise."
	
	| value |
	phidget isAttached ifFalse: [ ^ false ].
	value := aDouble asFloat.
	( value >= 0.0 and: [ value <= 1.0 ] ) ifFalse: [ ^ false ].

	^ (self class
		primSetChangeTrigger: phidget id
		at: index - 1
		to: value) == 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/25/2009 13:26'!
sensitivity
	sensitivity ifNil: [ sensitivity := 1.0 ].
	^ sensitivity! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/25/2009 13:27'!
sensitivity: aNumber
	sensitivity := aNumber.! !

PhdCrMultiComponent subclass: #PhdCrCompDigitalInput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompDigitalInput commentStamp: 'sp 7/1/2009 14:39' prior: 0!
I am a special Phidget Component for digital input values.

I can be asked for a value I am currently reading, either true or false (getter exists). True means there is a current I can read, false there is not.

My Phidget can call a user defined callback block when my value changes, when prior
	digitalInput on: #InputChange do: [ "user defined content" ]
was called.

Currently I belong to 
	- PhdCrInterfaceKit
	- PhdCrMotorControl.!

!PhdCrCompDigitalInput class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:56'!
primComponentCount: phidgetIdArray
	"Gets the number of inputs for a phidget digital input component."

	<primitive: 'primNumDigitalInputs' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalInput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetValue: phidgetIdArray at: index
	"Gets the input value (0/1 = off/on) at the digital input with the given index."

	<primitive: 'primGetDigitalInputValue' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalInput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an input value changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexDigitalInputChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompDigitalInput methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:53'!
value
	"Gets the current value of the input (true/false = on/off)."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primGetValue: phidget id
		at: index - 1) == 1! !

PhdCrMultiComponent subclass: #PhdCrCompDigitalOutput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompDigitalOutput commentStamp: 'sp 7/1/2009 14:39' prior: 0!
I am a special Phidget Component for digital output values.

I can be asked for or set a value I am currently having, either true or false (getter/setter exist). True means there is a current at this output, false that there is not.

My Phidget can call a user defined callback block when my value changes, when prior
	digitalOutput on: #OutputChange do: [ "user defined content" ]
was called.

Currently I can belong to
	- PhdCrInterfaceKit
	- PhdCrRFID!

!PhdCrCompDigitalOutput class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:56'!
primComponentCount: phidgetIdArray
	"Gets the number of outputs for a phidget digital output component."

	<primitive: 'primNumDigitalOutputs' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalOutput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetValue: phidgetIdArray at: index
	"Gets the output value (0/1 = off/on) at the digital output with the given index."

	<primitive: 'primGetDigitalOutputValue' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalOutput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an output value changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexDigitalOutputChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompDigitalOutput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primSetValue: phidgetIdArray at: index to: value
	"Sets the output value (0/1 = off/on) at the digital output with the given index to the given value.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetDigitalOutputValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompDigitalOutput methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:55'!
value
	"Gets the current value of the output (true/false = on/off)."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primGetValue: phidget id
		at: index - 1) == 1! !

!PhdCrCompDigitalOutput methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:55'!
value: aBoolean
	"Sets the current value of the output (true = on, false = off).
	Returns true if the value was sucessfully set, false otherwise."
	
	| number |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean
		ifTrue: [ number := 1 ]
		ifFalse: [ number := 0 ].
	^ (self class
		primSetValue: phidget id
		at: index - 1
		to: number) == 1! !

PhdCrMultiComponent subclass: #PhdCrCompMotor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompMotor commentStamp: 'lk 8/6/2009 22:01' prior: 0!
I am a special Phidget Component for motor controls.

I have a current acceleration, which can be retrieved and set (getter/setter exist). This acceleration should be between my minimal and maximal acceleration which can also be retrieved. Be careful that if those values cannot be retrieved (for instance if the Phidget is not attached) they return a value of 0.0, although this may also be a valid value.
In addition I have a current velocity, which can be retrieved and set (getter/setter exist). This velocity should be between -100.0 to 100.0. Setting other values will have no effect but returning false.
I also can be asked for my current current draw.

My Phidget can call a user defined callback block when my motor velocity changes, when prior
	motorControl on: #VelocityChange do: [ "user defined content" ]
was called.
It also can call a user defined callback block when the current draw of my motor changes, when prior
	motorControl on: #CurrentChange do: [ "user defined content" ]
was called.

Currently I only belong to PhdCrMotorControl.!

!PhdCrCompMotor class methodsFor: 'multi-component' stamp: 'sp 6/24/2009 14:29'!
primComponentCount: phidgetHandle
	"Gets the number of motors for a phidget motor component."

	<primitive: 'primNumMotors' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 6/24/2009 14:36'!
primGetAcceleration: phidgetHandle at: index
	"Gets the acceleration of the given motor."

	<primitive: 'primGetMotorAcceleration' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 7/1/2009 16:17'!
primGetAccelerationMax: phidgetHandle at: index
	"Returns a float containing max. possible acceleration of the motor at the given index."

	<primitive: 'primGetMotorAccelerationMax' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 7/1/2009 16:17'!
primGetAccelerationMin: phidgetHandle at: index
	"Returns a float containing min. possible acceleration of the motor at the given index."

	<primitive: 'primGetMotorAccelerationMin' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 7/1/2009 16:18'!
primGetCurrent: phidgetHandle at: index
	"Gets the current of the given motor."

	<primitive: 'primGetMotorCurrent' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 6/24/2009 14:48'!
primGetVelocity: phidgetHandle at: index
	"Gets the velocity of the given motor."

	<primitive: 'primGetMotorVelocity' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 6/24/2009 14:37'!
primSetAcceleration: phidgetIdArray at: index to: aFloat
	"Set the acceleration of the motor at the given index to aFloat. aFloat must be between primGetAccelerationMin and primGetAccelerationMax.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetMotorAcceleration' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 06:29'!
primSetSemaphoreIndexCurrentChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when the current draw changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexMotorCurrentChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 06:29'!
primSetSemaphoreIndexVelocityChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when the velocity changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexMotorVelocityChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor class methodsFor: 'primitives' stamp: 'sp 7/1/2009 16:36'!
primSetVelocity: phidgetIdArray at: index to: aFloat
	"Set the velocity of the motor at the given index to aFloat. (For succeed: aFloat has to be in Range of -100.0 and 100.0)
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetMotorVelocity' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 6/24/2009 14:33'!
acceleration
	"Gets the current acceleration of the motor (double between min and max).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAcceleration: phidget id
		at: index - 1! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 6/25/2009 21:45'!
acceleration: aValue
	"Set the acceleration of the motor.
	aValue should be a number betwenn self accelerationMin and self accelerationMax (inclusivly).
	Returns true if the setting was successful, false otherwise."
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	(self accelerationIsInRange: aFloat) ifFalse: [ ^ false ].
	^ (self class
		primSetAcceleration: phidget id
		at: index - 1
		to: aFloat) == 1! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 16:21'!
accelerationIsInRange: acceleration
	"Return true if the acceleration is in range of this motor, false otherwise.
	The acceleration is in range if the Phidget is attached and the acceleration is between the min. and max. acceleration."
	
	phidget isAttached ifFalse: [ ^ false ].
	self accelerationMin <= acceleration ifFalse: [ ^ false ].
	self accelerationMax >= acceleration ifFalse: [ ^ false ].
	^ true! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 6/24/2009 14:34'!
accelerationMax
	"Gets the max. acceleration supported by a motor
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAccelerationMax: phidget id
		at: index - 1! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 6/24/2009 14:33'!
accelerationMin
	"Gets the min. acceleration supported by a motor
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAccelerationMin: phidget id
		at: index - 1! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 6/24/2009 14:47'!
current
	"Gets the current current draw for a motor. 
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetCurrent: phidget id
		at: index - 1! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 6/24/2009 14:40'!
velocity
	"Gets the current velocity of the motor.
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetVelocity: phidget id
		at: index - 1! !

!PhdCrCompMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 16:34'!
velocity: aValue
	"Set the velocity of the motor. aValue should be in Range of -100.0 and 100.0.
	Returns true if the setting was successful, false otherwise."
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	^ (self class
		primSetVelocity: phidget id
		at: index - 1
		to: aFloat) == 1! !

PhdCrMultiComponent subclass: #PhdCrCompServoMotor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompServoMotor commentStamp: 'sp 7/1/2009 14:40' prior: 0!
I am a special Phidget Component for simple servo motors.

I have a current position, which can be retrieved and set (getter/setter exist). This position should be between my minimal and maximal position which can also be retrieved. Be careful that if those values cannot be retrieved (for instance if the Phidget is not attached) they return a value of 0.0, although this may also be a valid value.
My motor is automatically engaged when a new position is set. The boolean value whether it is engaged or not can be retrieved and set.

My Phidget can call a user defined callback block when my motor position changes, when prior
	servoMotor on: #PositionChange do: [ "user defined content" ]
was called.

Currently I only belong to PhdCrServo.!

PhdCrCompServoMotor subclass: #PhdCrCompAdvancedServoMotor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompAdvancedServoMotor commentStamp: 'sp 7/1/2009 16:02' prior: 0!
I am a special Phidget Component for advanced servo motors.

Like my super class I have a current position and my motor is engaged when a new position is set (see my super class' documentation for more information). Additionally I am able to limitate the min. and max. position of a motor.
I have a current velocity and acceleration, which can be retrieved and set (getter/setter exist). This velocity (or acceleration) should be between my minimal and maximal velocity (or acceleration) which can also be retrieved. Be careful that if those values cannot be retrieved (for instance if the Phidget is not attached) they return a value of 0.0, although this may also be a valid value. Furthermore I'm able to limitate the min. and max. velocity of a motor.
I also can be asked for my current current draw and whether I'm stopped or not.
In isSpinningAccelerated can be set whether I should keep attention on acceleration and velocity or not.

My Phidget can call a user defined callback block when my motor velocity changes, when prior
	advServoMotor on: #VelocityChange do: [ "user defined content" ]
was called.
It also can call a user defined callback block when the current draw of my motor changes, when prior
	advServoMotor on: #CurrentChange do: [ "user defined content" ]
was called.

Currently I only belong to PhdCrAdvancedServo.!

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetAcceleration: phidgetHandle at: index
	"Gets the acceleration of the given motor."

	<primitive: 'primGetAdvancedServoAcceleration' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetAccelerationMax: phidgetHandle at: index
	"Gets the max. acceleration of the given motor."

	<primitive: 'primGetAdvancedServoAccelerationMax' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetAccelerationMin: phidgetHandle at: index
	"Gets the min. acceleration of the given motor."

	<primitive: 'primGetAdvancedServoAccelerationMin' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetCurrent: phidgetHandle at: index
	"Gets the current of the given motor."

	<primitive: 'primGetAdvancedServoCurrent' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetVelocity: phidgetHandle at: index
	"Gets the velocity of the given motor."

	<primitive: 'primGetAdvancedServoVelocity' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetVelocityLimit: phidgetHandle at: index
	"Gets the limit of the velocity of the given motor."

	<primitive: 'primGetAdvancedServoVelocityLimit' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetVelocityMax: phidgetHandle at: index
	"Gets the max. velocity of the given motor."

	<primitive: 'primGetAdvancedServoVelocityMax' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:27'!
primGetVelocityMin: phidgetHandle at: index
	"Gets the min. velocity of the given motor."

	<primitive: 'primGetAdvancedServoVelocityMin' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:51'!
primIsSpeedRampingOn: phidgetIdArray at: index
	"Returns 1 if the motor on the given index is using acceleration and velocity, or other values otherwise."

	<primitive: 'primIsAdvancedServoSpeedRampingOn' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 18:15'!
primIsStopped: phidgetIdArray at: index
	"Returns 1 if the motor on the given index is stopped and that there are no commands in the pipeline
to the servo, or other values otherwise."

	<primitive: 'primIsAdvancedServoStopped' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:28'!
primSetAcceleration: phidgetIdArray at: index to: aFloat
	"Set the acceleration of the motor at the given index to aFloat. aFloat must be between primGetAccelerationMin and primGetAccelerationMax.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetAdvancedServoAcceleration' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:28'!
primSetPositionMax: phidgetHandle at: index
	"Sets the max. position of the given motor."

	<primitive: 'primGetAdvancedServoPositionMax' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:28'!
primSetPositionMin: phidgetHandle at: index
	"Sets the min. position of the given motor."

	<primitive: 'primGetAdvancedServoPositionMin' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 06:35'!
primSetSemaphoreIndexCurrentChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when the current draw changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAdvancedServoCurrentChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 06:35'!
primSetSemaphoreIndexVelocityChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when the velocity changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAdvancedServoVelocityChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 7/1/2009 16:31'!
primSetSpeedRampingOn: phidgetIdArray at: index to: value
	"Sets whether acceleration and velocity of the motor at the given index are used or not.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetAdvancedServoSpeedRampingOn' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 7/1/2009 16:38'!
primSetVelocity: phidgetIdArray at: index to: aFloat
	"Set the velocity of the motor at the given index to aFloat. aFloat must be between primGetVelocityMin and primGetVelocityMax.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetAdvancedServoVelocity' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor class methodsFor: 'primitives' stamp: 'sp 6/26/2009 17:28'!
primSetVelocityLimit: phidgetIdArray at: index to: aFloat
	"Set the limit of the velocity of the motor at the given index to aFloat. aFloat must be between primGetVelocityMin and primGetVelocityMax.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetAdvancedServoVelocityLimit' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 17:20'!
acceleration
	"Gets the current acceleration of the motor (double between min and max).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAcceleration: phidget id
		at: index - 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 17:20'!
acceleration: aValue
	"Set the acceleration of the motor.
	aValue should be a number betwenn self accelerationMin and self accelerationMax (inclusivly).
	Returns true if the setting was successful, false otherwise."
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	(self accelerationIsInRange: aFloat) ifFalse: [ ^ false ].
	^ (self class
		primSetAcceleration: phidget id
		at: index - 1
		to: aFloat) == 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 16:30'!
accelerationIsInRange: acceleration
	"Return true if the acceleration is in range of this motor, false otherwise.
	The acceleration is in range if the Phidget is attached and the acceleration is between the min. and max. acceleration."
	
	phidget isAttached ifFalse: [ ^ false ].
	self accelerationMin <= acceleration ifFalse: [ ^ false ].
	self accelerationMax >= acceleration ifFalse: [ ^ false ].
	^ true! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 17:20'!
accelerationMax
	"Gets the max. acceleration supported by a motor
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAccelerationMax: phidget id
		at: index - 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 17:21'!
accelerationMin
	"Gets the min. acceleration supported by a motor
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetAccelerationMin: phidget id
		at: index - 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 17:22'!
current
	"Gets the current current draw for a motor. 
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetCurrent: phidget id
		at: index - 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 15:48'!
isSpinningAccelerated
	"Returns the speed ramping state of the motor. This is whether or not velocity and acceleration are used."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primIsSpeedRampingOn: phidget id
		at: index - 1) == 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 15:48'!
isSpinningAccelerated: aBoolean
	"Sets the speed ramping state of a motor. This is whether or not velocity and acceleration are used.
	Returns true if the setting was successful, false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetSpeedRampingOn: phidget id
		at: index - 1
		to: anInt) == 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 18:18'!
isStopped
	"Returns the motion state of the motor. This is whether the motor has stopped or not."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primIsStopped: phidget id
		at: index - 1) == 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 17:04'!
positionIsInRange: position
	"Return true if the position is in range of this motor, false otherwise.
	The position is in range if the Phidget is attached and the position is between the original min. and max. position."
	
	phidget isAttached ifFalse: [ ^ false ].
	( -22.9921875 <= position and: [ 233.0 >= position ]) ifFalse: [ ^ false ].
	^ true! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 17:15'!
positionMax: aValue
	"Sets the max. allowed position of the motor (double between min and max).
	Returns true if the setting was successful, false otherwise."
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	(self positionIsInRange: aFloat) ifFalse:[ ^ false ].
	^( self class
		primSetPostionMax: phidget id
		at: index - 1 
		to: aFloat) == 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 16:46'!
positionMin: aValue
	"Sets the min. allowed position of the motor (double between original min and max).
	Returns true if the setting was successful, false otherwise."
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	(self positionIsInRange: aFloat) ifFalse:[ ^ false ].
	^( self class
		primSetPostionMin: phidget id
		at: index - 1 
		to: aFloat) == 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 16:38'!
velocity
	"Gets the velocity of the motor (double between min and max with regard to the limit).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetVelocity: phidget id
		at: index - 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 16:30'!
velocityIsInRange: velocity
	"Return true if the velocity is in range of this motor, false otherwise.
	The velocity is in range if the Phidget is attached and the velocity is between the min. and max. velocity."
	
	phidget isAttached ifFalse: [ ^ false ].
	self velocityMin <= velocity ifFalse: [ ^ false ].
	self velocityMax >= velocity ifFalse: [ ^ false ].
	^ true! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 16:54'!
velocityLimit
	"Gets the limited velocity of the motor (double between min and max).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetVelocityLimit: phidget id
		at: index - 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 7/1/2009 16:56'!
velocityLimit: aValue
	"Sets the limit of the velocity of the motor (double between min and max).
	if there was an error, return flase"
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	(self velocityIsInRange: aFloat) ifFalse:[ ^ false ].
	^( self class
		primSetVelocityLimit: phidget id
		at: index - 1 
		to: aFloat) == 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 16:39'!
velocityMax
	"Gets the max. velocity supported by a motor.
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetVelocityMax: phidget id
		at: index - 1! !

!PhdCrCompAdvancedServoMotor methodsFor: 'accessing' stamp: 'sp 6/26/2009 16:39'!
velocityMin
	"Gets the min. velocity supported by a motor.
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetVelocityMin: phidget id
		at: index - 1! !

!PhdCrCompServoMotor class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:56'!
primComponentCount: phidgetIdArray
	"Gets the number of servo motors for a (simple) servo motor component."

	<primitive: 'primNumServoMotors' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetMaxPosition: phidgetIdArray at: index
	"Returns a float containing max. possible position of the motor at the given index."

	<primitive: 'primGetServoPositionMax' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetMinPosition: phidgetIdArray at: index
	"Returns a float containing min. possible position of the motor at the given index."

	<primitive: 'primGetServoPositionMin' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetPosition: phidgetIdArray at: index
	"Returns a float containing the position of the motor at the given index."

	<primitive: 'primGetServoPosition' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primIsEngaged: phidgetIdArray at: index
	"Returns 1 if the motor on the given index is engaged, or other values otherwise."

	<primitive: 'primIsServoEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primSetEngaged: phidgetIdArray at: index to: value
	"Sets the engagement state (0/1 = off/on) of the motor at the given index.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetServoEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primSetPosition: phidgetIdArray at: index to: aFloat
	"Set the position of the motor at the given index to aFloat. aFloat must be between primGetPositionMin and primGetPositionMax.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetServoPosition' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primSetSemaphoreIndexPositionChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when a motor changes its position.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexServoPositionChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 15:10'!
isEngaged
	"Returns the engaged state of the motor. This is whether the motor is powered or not."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primIsEngaged: phidget id
		at: index - 1) == 1! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:12'!
isEngaged: aBoolean
	"Sets the engaged state of a motor. This is whether the motor is powered or not.
	Returns true if the setting was successful, false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetEngaged: phidget id
		at: index - 1
		to: anInt) == 1! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:10'!
position
	"Returns the current position of the motor as a floating point number.
	This value is between self positinMin and self positionMax.
	If it cannot be retrieved it is 0.0 (although this may also be a valid value)."
	
	| result |
	phidget isAttached ifFalse: [ ^ 0.0 ].
	result := self class
		primGetPosition: phidget id
		at: index - 1.
	^ result
		ifNotNil: [ result ]
		ifNil: [ 0.0 ]! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:11'!
position: aValue
	"Set the position of the motor.
	aValue should be a number betwenn self positionMin and self positionMax (inclusivly).
	Returns true if the setting was successful, false otherwise."
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	(self positionIsInRange: aFloat) ifFalse: [ ^ false ].
	^ (self class
		primSetPosition: phidget id
		at: index - 1
		to: aFloat) == 1! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:12'!
positionIsInRange: position
	"Return true if the position is in range of this motor, false otherwise.
	The position is in range if the Phidget is attached and the position is between the min. and max. position."
	
	phidget isAttached ifFalse: [ ^ false ].
	self positionMin <= position ifFalse: [ ^ false ].
	self positionMax >= position ifFalse: [ ^ false ].
	^ true! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:12'!
positionMax
	"Returns the max. allowed motor position for this motor.
	If it cannot be retrieved it is 0.0 (although this may also be a valid value)."
	
	| result |
	phidget isAttached ifFalse: [ ^ 0.0 ].
	result := self class
		primGetMaxPosition: phidget id
		at: index - 1.
	^ result
		ifNotNil: [ result ]
		ifNil: [ 0.0 ]! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:13'!
positionMin
	"Returns the min. allowed motor position for this motor.
	If it cannot be retrieved it is 0.0 (although this may also be a valid value)."
	
	| result |
	phidget isAttached ifFalse: [ ^ 0.0 ].
	result := self class
		primGetMinPosition: phidget id
		at: index - 1.
	^ result
		ifNotNil: [ result ]
		ifNil: [ 0.0 ]! !

!PhdCrCompServoMotor methodsFor: 'prototyp' stamp: 'pt 6/23/2009 18:50'!
positionNumber: aNumber

	| max min |
	( 0 <= aNumber and: [ aNumber <= 1000 ] ) ifFalse: [ ^ self ].
	max := self positionMax.
	min := self positionMin.
	min == -1000 ifFalse: [
		self position: max - min * (aNumber asFloat) / 1000.0 + min ].! !

!PhdCrMultiComponent class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:39'!
componentCount: phidget
	"Returns the number of components in a multicomponent phidget.
	Uses a primitive the subclasses must define. This primitive can for example do a call to get the number of sensors on an InterfaceKit.
	If there was an error getting the value, 0 is returned."
	
	phidget isAttached ifTrue: [ | result |
		result := self primComponentCount: phidget id.
		result ifNil: [ ^ 0 ].
		^ result ].
	 ^ 0! !

!PhdCrMultiComponent class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:40'!
getListFor: phidget
	"Answers a LinkedList of instances of my class, with self componentCount elements, each initialized with its index in the list."

	| list size |
	list := LinkedList new.
	size := self componentCount: phidget.
	
	(1 to: size) do: [ :i |
		list add: (self newWith: phidget index: i) ].
	^ list! !

!PhdCrMultiComponent class methodsFor: 'instance creation' stamp: 'lk 6/16/2009 20:38'!
newWith: phidget index: index
	"Create and return a new MultComponent with given Phidget 'parent' and index.
	Once those values are set, they are never changed again."

	^ (self newWith: phidget)
		setIndex: index! !

!PhdCrMultiComponent class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:52'!
primComponentCount: phidgetIdArray
	"Gets the number of components for a phidget multi component.
	This can for instance be a primitive to get the number of sensors on an InterfaceKit."

	self subclassResponsibility.! !

!PhdCrMultiComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 18:38'!
nextLink

	^ nextLink! !

!PhdCrMultiComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 18:38'!
nextLink: aMultiComponent

	nextLink := aMultiComponent.! !

!PhdCrMultiComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 18:17'!
setIndex: aNumber
	"Set my 'index' instance variable to the given one.
	If I already have an index, do nothing."
	
	index ifNil: [ index := aNumber ].! !

Object subclass: #PhdCrPhidget
	instanceVariableNames: 'opened semaphores userProcessBlocks id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

PhdCrPhidget class
	instanceVariableNames: 'SystemCallbacks'!

!PhdCrPhidget commentStamp: 'lk 8/5/2009 22:17' prior: 0!
I am an abstract class that provides some primitives and methods for Phidget support. Please read the comments of my subclasses to find more information how to use Phidgets in Squeak.

Instance Variables
	id:						<ByteArray>
	opened:					<Boolean>
	userProcessBlocks:	<Dictionary> (Symbol -> Block)
	semaphores:				<Dictionary> (Symbol -> Semaphore)
Class Instance Variables
	SystemCallback:		<Dictionary> (Symbol -> Symbol)
	
id
	- a ByteArray that contains the Phidget's unique identifier for the C API. Every primitive (except the one where the handle is created) needs this ByteArray to let the C API know about which Phidget it is currently talking to.
	- the id is obtained when creating the phidget object and will stay there until the object is deleted or the id became invalid (see checkInvalid method).
	- an id can become invalid if the phidget is created, the Squeak image saved, closed and then reloaded. As the plugin was shutdown when the VM is closed, the hardware connection to the phidget is broken. checkValid checks the id and resets it to a default value if the id has become invalid. This object cannot be used for communication with any Phidget anymore.
	- checkValid should be called before doing any primitve with a phidget id.

opened
	- true, if the Phidget was opened - Squeak is able to communicate with the hardware. This does not mean there is a Phidget attached.
	- false, if the Phidgets was not opened or was opened and then closed.

semaphores
	- A Dictionary with entries of the following form: aSymbol -> aSemaphore.
	aSymbol may be #Attach, #Detach, #Error for Phidgets in general. Other symbols may exist for phidget specific support in my subclasses. My classes method 'knownCallback' answers a list of callbacks I know.
	I use this dictionary to handle the callbacks from the Phidget C API.
	These semaphores can be registered for certain events (like the attach event) - so that the C API signals is if the event occours. These semaphores are used in my callbackProcess method.

userProcessBlocks
	- a Dictionary with entries of the following form: aSymbol -> aBlock.
	The symbols are equivalent to the symbols used in the semaphores dictionary.
	I use this dictionary to handle the callbacks from the Phidget C API.
	Each time a semaphore from the semaphores dictionary was signaled from the C API, I evaluate the corresponding block from this dictionary in the callbackProcess method.
	In the base class possible callbacks are: #Attach, #Detach, #Error

SystemCallbacks
	My class has a method called 'systemCallbacks'. This Dictionary is similar to the userProcessBlocks: It maps from callbackNames to message selectors. Those messages willbe sent when the semaphore at 'callbackName' is signaled.
	In contrast to userProcessBlocks this Dictionary is immutable to the user. Currently it is only used for initializing and deleting phidget components an attaching/detaching (see below).

About Components:
	A phidget can have several components, e.g. a list of outputs or a LED. Those components derive from the PhdCrComponent class. See its documentation to lern more about them.
	When a phidget is attached, it initializes its components using 'initializeComponents'. When the phidget is detached or closed, 'deleteComponents' deletes them again.
	A Component a Phidget can have is represented by an instance variable in that class. Add a getter and initialisation/deletion instructions in initialize / initializeComponents / deleteComponents.
	As there are different Phidgets per phidget class, the umber of components can differ. A InterfaceKit can for instance have 8 sensors, others might have none. The implementations of initializeComponents, delete components and initialize must be aware of that.

IMPORTANT:
After using the Phidget it is important to close the connection to the hardware by sending 'close':
	phidget close.
or to unload the plugin (e.g. by closing Squeak).!

PhdCrPhidget class
	instanceVariableNames: 'SystemCallbacks'!

PhdCrPhidget subclass: #PhdCrAccelerometer
	instanceVariableNames: 'axes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrAccelerometer commentStamp: 'sp 7/1/2009 12:34' prior: 0!
I am a representation of a Phidget called 'Accelerometer' in Squeak.
To use a Accelerometer, create a new instance and open a connection to the hardware board with:
	accel := PhdCrInterfaceKit new.
	accel openAny.
Send me a block to be evaluated every time the hardware board was attached, detached, an error or acceleration occoured.
For example:
	accel
		on: #Attach do: [ Transcript show:'accel was attached!!';cr ];
		on: #Detach do: [ Transcript show:'accel was detached!!';cr];
		on: #AccelerationChange do: [ Transcript show:'A axis changed it's value!!' ].


Instance Variables
	axes:		<LinkedList of PhdCrCompDigitalOutput's>

axes:
	- a LinkedList. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrComponent's as the Phidget has. When the Phidget is detached the list is emptied. See the instance's class documentation for more information.


new callbacks:
	#AccelerationChange (an axis has occured an acceleration)!

!PhdCrAccelerometer class methodsFor: 'callbacks' stamp: 'sp 6/18/2009 17:42'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Accelerometer knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #AccelerationChange )! !

!PhdCrAccelerometer class methodsFor: 'primitives' stamp: 'sp 6/18/2009 17:43'!
primCreate
	"Create a new the Acceleration object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primAccelerometerCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrAccelerometer methodsFor: 'accessing' stamp: 'sp 6/18/2009 17:45'!
axes
	^ axes! !

!PhdCrAccelerometer methodsFor: 'accessing' stamp: 'pt 6/26/2009 13:22'!
changeSensitivity
	| aNumber |
	
	self axes size = 0 ifTrue: [^ 0.0].
	self axes do: [:axis | aNumber := aNumber + axis sensitivity].
	^ aNumber asFloat / self axes size asFloat! !

!PhdCrAccelerometer methodsFor: 'accessing' stamp: 'pt 6/26/2009 13:22'!
changeTrigger 
	| aNumber |
	
	self axes size = 0 ifTrue: [^ 0.0].
	self axes do: [:axis | aNumber := aNumber + axis changeTrigger].
	^ aNumber asFloat / self axes size asFloat! !

!PhdCrAccelerometer methodsFor: 'accessing' stamp: 'lk 7/20/2009 16:05'!
changeTrigger: aNumber
	self axes do: [:axis | axis changeTrigger: aNumber].! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'sp 6/18/2009 17:40'!
deleteComponents
	"This method is called when the Accelerometer is detached. It delets previously in 'initializeComponents' built component axes."

	axes := LinkedList new.! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'sp 6/18/2009 17:36'!
initialize
	"additionally reset my components."

	super initialize.
	
	axes := LinkedList new.! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 21:53'!
initializeComponents
	"This method is called when an Accelerometer is attached. It initializes its component axes.
	The component will be deleted in 'deleteComponents' on detaching."

	axes := PhdCrCompAxis getListFor: self.
	
	super initializeComponents.! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'sp 6/18/2009 21:19'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for
	#AxisChange."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #AccelerationChange).
	PhdCrCompAxis
		primSetSemaphoreIndexChange: semaIndex
		for: id.! !

!PhdCrAccelerometer methodsFor: 'accessing' stamp: 'pt 6/26/2009 11:23'!
sensitivity: aNumber
 self axes do: [:axis | axis sensitivity: aNumber].! !

PhdCrPhidget subclass: #PhdCrInterfaceKit
	instanceVariableNames: 'sensors inputs outputs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrInterfaceKit commentStamp: 'sp 7/1/2009 12:35' prior: 0!
I am a representation of a Phidget called 'Interface Kit' in Squeak.
To use a Interface Kit, create a new instance and open a connection to the hardware board with:
	ifKit := PhdCrInterfaceKit new.
	ifKit openAny.
Send me a block to be evaluated every time the hardware board was attached, detached, an error occoured, a sensor value was changed, an input value was changed or an output value was changed. For example:
	ifKit
		on: #Attach do: [ Transcript show:'ifKit was attached!!';cr ];
		on: #Detach do: [ Transcript show:'ifKit was detached!!';cr ];
		on: #SensorChange do: [ Transcript show:'A sensor changed it's value!!' ].


Instance Variables
	sensors:		<LinkedList of PhdCrCompAnalogSensor's>
	inputs:		<LinkedList of PhdCrCompDigitalInput's>
	outputs:		<LinkedList of PhdCrCompDigitalOutput's>

sensors / inputs / outputs: (each)
	- a LinkedList. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrComponent's as the Phidget has. When the Phidget is detached the list is emptied. See the instance's class documentation for more information.


new callbacks:
	#SensorChange (a sensor has changed its value),
	#InputChange (an input has changed its value),
	#OutputChange (an output has changed its value)
!

!PhdCrInterfaceKit class methodsFor: 'callbacks' stamp: 'lk 5/12/2009 12:23'!
knownCallbacks
	"Returns a list of callback names (#symbols) the InterfaceKit knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #SensorChange #InputChange #OutputChange )! !

!PhdCrInterfaceKit class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:17'!
primCreate
	"Create a new the Interface Kit object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primInterfaceKitCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 13:36'!
deleteComponents
	"This method is called when the InterfaceKit is detached. It delets previously in 'initializeComponents' built components: sensors, inputs, outputs."

	sensors := LinkedList new.
	inputs := LinkedList new.
	outputs := LinkedList new.! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:04'!
initialize
	"additionally reset my components."

	super initialize.
	
	inputs := LinkedList new.
	outputs := LinkedList new.
	sensors := LinkedList new.! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 21:56'!
initializeComponents
	"This method is called when an InterfaceKit is attached. It initializes its components: sensors, inputs, outputs.
	The components will be deleted in 'deleteComponents' on detaching."
	
	sensors := PhdCrCompAnalogSensor getListFor: self.
	inputs := PhdCrCompDigitalInput getListFor: self.
	outputs := PhdCrCompDigitalOutput getListFor: self.
	
	super initializeComponents.! !

!PhdCrInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:13'!
inputs
	^ inputs! !

!PhdCrInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:13'!
outputs
	^ outputs! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 16:07'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for #SensorChange, #InputChange and #OutputChange."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #SensorChange).
	PhdCrCompAnalogSensor
		primSetSemaphoreIndexChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #InputChange).
	PhdCrCompDigitalInput
		primSetSemaphoreIndexChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #OutputChange).
	PhdCrCompDigitalOutput
		primSetSemaphoreIndexChange: semaIndex
		for: id.! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:17'!
resetState
	"when closing/attaching, reset all outputs to false."

	super resetState.	
	self outputs do: [ :output | output value: false ].! !

!PhdCrInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:14'!
sensors
	^ sensors! !

PhdCrPhidget subclass: #PhdCrMotorControl
	instanceVariableNames: 'motors inputs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrMotorControl commentStamp: 'sp 7/1/2009 13:03' prior: 0!
I am a representation of a Phidget called 'Motor Control' in Squeak.
To use a Motor Control, create a new instance and open a connection to the hardware board with:
	motorControl := PhdCrMotorControl new.
	motorControl openAny.
Send me a block to be evaluated every time the hardware board was attached, detached, an error occoured, the velocity of a motor was changed, the current of a motor was changed or an input value was changed.
For example:
	motorControl
		on: #Attach do: [ Transcript show:'motorControl was attached!!';cr ];
		on: #Detach do: [ Transcript show:'motorControl was detached!!';cr ];
		on: #VelocityChange do: [ Transcript show:'A motor changed it's velocity!!' ];
		on: #CurrentChange do: [ Transcript show:'The current draw for a motor has changed!!' ].


Instance Variables
	motors:		<LinkedList of PhdCrCompMotor's>
	inputs:		<LinkedList of PhdCrCompDigitalInput's>

motors / inputs: (each)
	- a LinkedList. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrComponent's as the Phidget has. When the Phidget is detached the list is emptied. See the instance's class documentation for more information.


new callbacks:
	#VelocityChange (a motor has changed its velocity),
	#CurrentChange (a motor has changed its current draw),
	#InputChange (an input has changed its value)!

!PhdCrMotorControl class methodsFor: 'callbacks' stamp: 'sp 6/24/2009 14:18'!
knownCallbacks
	"Returns a list of callback names (#symbols) the MotorControl knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #VelocityChange #CurrentChange #InputChange )! !

!PhdCrMotorControl class methodsFor: 'primitives' stamp: 'sp 6/24/2009 14:17'!
primCreate
	"Create a new the Motor Control object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primMotorControlCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrMotorControl methodsFor: 'initialize-open-close' stamp: 'sp 6/24/2009 13:46'!
deleteComponents
	"This method is called when the MotorControl is detached. It delets previously in 'initializeComponents' built components: motors and inputs."

	motors := LinkedList new.
	inputs := LinkedList new.! !

!PhdCrMotorControl methodsFor: 'initialize-open-close' stamp: 'sp 6/24/2009 13:40'!
initialize
	"additionally reset my components."

	super initialize.
	
	motors := LinkedList new.
	inputs := LinkedList new.! !

!PhdCrMotorControl methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 21:56'!
initializeComponents
	"This method is called when a MotorControl is attached. It initializes its components: motors and inputs.
	The components will be deleted in 'deleteComponents' on detaching."
	
	motors := PhdCrCompMotor getListFor: self.
	inputs := PhdCrCompDigitalInput getListFor: self.
	
	super initializeComponents.! !

!PhdCrMotorControl methodsFor: 'accessing' stamp: 'sp 6/24/2009 13:51'!
inputs
	^ inputs! !

!PhdCrMotorControl methodsFor: 'accessing' stamp: 'sp 6/24/2009 13:50'!
motors
	^ motors! !

!PhdCrMotorControl methodsFor: 'initialize-open-close' stamp: 'sp 6/25/2009 13:26'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for #VelocityChange, #CurrentChange and #InputChange"
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #VelocityChange).
	PhdCrCompMotor
		primSetSemaphoreIndexVelocityChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #CurrentChange).
	PhdCrCompMotor
		primSetSemaphoreIndexCurrentChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #InputChange).
	PhdCrCompDigitalInput
		primSetSemaphoreIndexChange: semaIndex
		for: id.! !

!PhdCrMotorControl methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:18'!
resetState
	"when closing/attaching, reset all motors to be stopped."

	super resetState.	
	self motors do: [ :motor |
		motor acceleration: 0.0.
		motor velocity: 0.0 ].! !

!PhdCrPhidget class methodsFor: 'callbacks' stamp: 'lk 6/16/2009 21:14'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Phidget knows. A subclass may extend this list if needed."
	
	^ #( #Attach #Detach #Error )! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:15'!
primClose: phidgetidArray
	"Close the Phidget with given id. 
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primClose' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 8/5/2009 18:15'!
primGetSerialNumber: phidgetidArray
	"Returns the Phidget's serial number, or -1 if there was an error.."
	
	<primitive: 'primGetSerialNumber' module: 'PhidgetPlugin'>
	^ -2! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:14'!
primIsAttached: phidgetidArray
	"Returns 1 if the Phidget is attached, and other values otherwise."
	
	<primitive: 'primIsAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:15'!
primIsValid: phidgetidArray
	"Returns 1 if the Phidget id is valid, or other values otherwise."
	
	<primitive: 'primIsValid' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 7/8/2009 15:45'!
primOpen: phidgetidArray serial: serial serverAddress: serverAddress port: port password: password
	"Opens the Phidget remotely with the given serial number, server address, port and password.
	Return 1 if the operation succeeded, or other values otherwise."

	<primitive: 'primOpenRemoteAddress' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 7/8/2009 14:56'!
primOpen: phidgetidArray serial: serial serverID: serverID password: password
	"Opens the Phidget remotely with the given serial number, server ID and password.
	Return 1 if the operation succeeded, or other values otherwise."

	<primitive: 'primOpenRemoteID' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primOpen: phidgetidArray withSerial: serial
	"Opens the Phidget with the given serial number.
	Return 1 if the operation succeeded, or other values otherwise."

	<primitive: 'primOpen' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primSetSemaphoreIndexAttach: semaphoreIndex for: phidgetidArray
	"registers the given Semaphore to be signaled when the Phidget with given id is attached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAttach' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primSetSemaphoreIndexDetach: semaphoreIndex for: phidgetidArray
	"registers the given Semaphore to be signaled when the Phidget with given id is attached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexDetach' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primSetSemaphoreIndexError: semaphoreIndex for: phidgetidArray
	"registers the given Semaphore to be signaled when the Phidget with given id is attached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'callbacks' stamp: 'lk 7/8/2009 18:15'!
systemCallbacks
	"This Method answers a Dictionary similar to the instance's userProcessBlocks.
	It may for instance contain message selectors sent when attaching or detaching a phidget, so the phidget components can be initialized."
	
	SystemCallbacks ifNotNil: [ ^ SystemCallbacks ].
	
	SystemCallbacks := Dictionary new.
	SystemCallbacks
		add: #Attach -> #initializeComponents;
		add: #Detach -> #deleteComponents.
	^ SystemCallbacks! !

!PhdCrPhidget methodsFor: 'callbacks' stamp: 'pt 7/14/2009 17:03'!
callbackProcess: callbackName 
	"This method will be forked as an extra process for every callback that a phidget needs.
	The semaphores were previously initialised and registered as external objects.
	self class knownCallbacks lists all possible semaphore names.
	systemCallbacks contains message selectors to be sent."
	
	(semaphores at: callbackName) wait.
	
	[ self isOpen ] whileTrue: [
		(self class systemCallbacks includesKey: callbackName) 
			ifTrue: [ self perform: (self class systemCallbacks at: callbackName) ].
		(userProcessBlocks at: callbackName) value.
		
		(semaphores at: callbackName) wait
	].! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 8/5/2009 16:51'!
checkValid
	"Checks if the Phidgets has an id and if this id is valid."

	^ id isNil not and: [ | result |
		result := (self class primIsValid: id) == 1.
		result ifFalse: [
			"this may have happened if a Phidget was created and then the image was saved,
			closed and reloaded. The hardware connection is broken. We need to make this
			instance fully invalid."
			id := nil.
			self doPostCloseOperations ].
		result ]! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 21:48'!
close
	"Close the Phidget."
	
	self isOpen ifFalse: [ ^ self ].	"already closed"
	
	self resetState.
	self class primClose: id.
	self doPostCloseOperations.! !

!PhdCrPhidget methodsFor: 'callbacks' stamp: 'lk 6/1/2009 00:47'!
createCallbackProcessBlock: callbackName
	"Create a callback process block to be forked.
	Must be swapped out to work in loops."

	^ [ self callbackProcess: callbackName ]! !

!PhdCrPhidget methodsFor: 'private' stamp: 'lk 6/1/2009 12:41'!
deleteComponents
	"This method is called when a Phidget is detached. It should delete previously in 'initializeComponents' built components, e.g. outputs, sensors, ...
	The abstract phidget does not have components, so it does nothing here."! !

!PhdCrPhidget methodsFor: 'private' stamp: 'lk 8/5/2009 16:51'!
doPostCloseOperations
	"Reset state when the Phidget has been closed,"

	self
		opened: false;
		unregisterSemaphores;
		deleteComponents.! !

!PhdCrPhidget methodsFor: 'callbacks' stamp: 'lk 6/1/2009 12:05'!
forkCallbacks
	"Fork the callback processes."
	
	self class knownCallbacks do: [ :callbackName |
		(self createCallbackProcessBlock: callbackName) fork ].! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 6/15/2009 11:24'!
id
	^ id! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'sp 7/18/2009 21:13'!
initialize
	"Initialise the callbacks that are used in every Phidget.
	Every subclass should call super initialise or initialise the callbacks by their own.
	In addition mark phidget as closed and create the representation in the plugin using a primitive."
	
	super initialize.
	
	semaphores := Dictionary new.
	userProcessBlocks := Dictionary new.
	opened := false.
	
	self class knownCallbacks do: [ :name | 
		semaphores add: name -> Semaphore new.
		userProcessBlocks add: name -> [] ].
	
	id := self class primCreate.! !

!PhdCrPhidget methodsFor: 'private' stamp: 'lk 8/7/2009 12:30'!
initializeComponents
	"This method is called when a Phidget is attached. It should initialize its components (sensors, outputs, ...).
	The components will be deleted in 'deleteComponents' on detaching.
	The abstract phidget does not have components, so it does nothing here."
	
	self resetState.! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 6/15/2009 20:34'!
isAttached
	"Returns true if the Phidget is attached, and false otherwise."

	^ self checkValid & opened and: [ (self class primIsAttached: id) == 1 ]! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 6/16/2009 21:09'!
isOpen
	"return true if I am open (having a hardware connection to a Phidget or waitng for one).
	Also make sure I am sill valid."

	^ self checkValid and: [ opened ]! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 6/16/2009 21:09'!
on: callbackName do: aBlock 
	"aBlock will be evaluated when the semaphore at callbackName is signaled.
	self class knownCallbacks lists all possible callback names."
	
	(self class knownCallbacks includes: callbackName) ifTrue: [
		userProcessBlocks 
			at: callbackName
			put: aBlock ].! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 13:09'!
openAny
	"opens hardware connection to any phidget with matching class."

	^ self openWithSerial: -1! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 7/15/2009 15:17'!
openRemoteWithSerial: serialNumber serverAddress: serverAddress port: port
	"Opens the Phidget remotely with the given serial number, serverAddres, port and without a password.
	Opens any Phidgets of matching class if serialNumber is -1."

	^ self
		openRemoteWithSerial: serialNumber
		serverAddress: serverAddress
		port: port
		password: nil! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:17'!
openRemoteWithSerial: serialNumber serverAddress: serverAddress port: port password: password
	"Opens the Phidget remotely with the given serial number, serverAddres, port and password.
	Opens any Phidgets of matching class if serialNumber is -1."

	| result passwordString |
	(self checkValid and: [self isOpen not]) ifFalse: [ ^ self ].
	
	serialNumber isInteger & serverAddress isString & port isInteger
		ifFalse: [ ^ self ].
	passwordString := [password isNil not and: [ password isString ]]
		ifTrue: [ password ]
		ifFalse: [ '' ].
	
	self registerSemaphores.
	result := (self class
		primOpen: id
		serial: serialNumber
		serverAddress: serverAddress
		port: port
		password: passwordString) == 1.
	
	result
		ifTrue: [
			self opened: true.
			self forkCallbacks ]
		ifFalse: [ self unregisterSemaphores ].! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:17'!
openRemoteWithSerial: serialNumber serverID: serverID password: password
	"Opens the Phidget remotely with the given serial number, serverID and password.
	Opens any Phidgets of matching class if serialNumber is -1."
	"DOES NOT WORK YET"

	"| result |
	(self checkValid and: [self isOpen not]) ifFalse: [ ^ self ].
	
	serialNumber isInteger & serverID isString
		ifFalse: [ ^ self ].
	
	self registerSemaphores.
	result := (self class
		primOpen: id
		serial: serialNumber
		serverID: serverID
		password: password) == 1.
	
	result
		ifTrue: [
			self opened: true.
			self forkCallbacks ]
		ifFalse: [ self unregisterSemaphores ]."! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 7/15/2009 15:16'!
openRemoteWithServerAddress: serverAddress port: port
	"Opens any Phidget remotely with matching type, serverAddres, port and without a password."

	^ self
		openRemoteWithSerial: -1
		serverAddress: serverAddress
		port: port! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 7/15/2009 15:15'!
openRemoteWithServerAddress: serverAddress port: port password: password
	"Opens any Phidget remotely with matching type, serverAddres, port and password."

	^ self
		openRemoteWithSerial: -1
		serverAddress: serverAddress
		port: port
		password: password! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:17'!
openWithSerial: serialNumber 
	"Opens the Phidget with the given serial number.
	Opens any Phidgets of matching class if serialNumber is -1."

	| result |
	(self checkValid and: [self isOpen not]) ifFalse: [ ^ self ].
	
	serialNumber isInteger ifFalse: [ ^ self ].
	
	self registerSemaphores.
	result := (self class
		primOpen: id
		withSerial: serialNumber) == 1.
		
	result
		ifTrue: [
			self opened: true.
			self forkCallbacks ]
		ifFalse: [ self unregisterSemaphores ].! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 8/5/2009 22:19'!
opened: aBoolean

	opened == aBoolean ifTrue: [ ^ self ].
	opened := aBoolean.
	aBoolean
		ifTrue: [ PhdCrPhidgetManager addToOpenedPhidgets: self ]
		ifFalse: [ PhdCrPhidgetManager removeFromOpenedPhidgets: self ].! !

!PhdCrPhidget methodsFor: 'private' stamp: 'lk 6/15/2009 12:32'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. May be extended in subclasses."
	
	| semaIndex |
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #Attach).
	self class
		primSetSemaphoreIndexAttach: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #Detach).
	self class
		primSetSemaphoreIndexDetach: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #Error).
	self class
		primSetSemaphoreIndexError: semaIndex
		for: id.! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:17'!
resetState
	"resets the Phidget state on closing/attaching of the Phidget."! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 8/5/2009 18:06'!
serialNumber
	"Return the Phidget's serial number. If there was an error, -1 is returned."

	self checkValid ifFalse: [ ^ -1 ].
	^ self class primGetSerialNumber: id! !

!PhdCrPhidget methodsFor: 'testing' stamp: 'lk 6/16/2009 21:10'!
testSemaphoresReset
	"Answer true if the semaphores are reset, false otherwise."

	id do: [ :byte | byte isZero ifFalse: [ ^ false ] ].
	^ true! !

!PhdCrPhidget methodsFor: 'private' stamp: 'pt 7/14/2009 16:21'!
unregisterSemaphores
	"Unregister the semaphores which were previously registered with 'self registerSemaphores'."

	self opened: false.
	semaphores do: [ :sema |
		"If I am not opened and the semaphore is signaled, the callback process will terminate."
		sema signal.
		
		Smalltalk unregisterExternalObject: sema
	].! !

PhdCrPhidget subclass: #PhdCrRFID
	instanceVariableNames: 'rfid led outputs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrRFID commentStamp: 'lk 6/16/2009 16:39' prior: 0!
I am a representation of a Phidget called 'Phidget RFID' in Squeak.
To use the RFID Phidget, create a new instance and open a connection to the hardware board with:
	rfid := PhdCrRFID new.
	rfid openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured, an output value was changed, a tag was found by the RFID reader, or if the tag was lost. For example:
	rfid
		on: #Attach do: [ Transcript show:'rfid was attached!!';cr ];
		on: #Detach do: [ Transcript show:'rfid was detached!!';cr ];
		on: #TagFound do: [ Transcript show:'rfid found a tag';cr ].


Instance Variables
	rfid:			<PhdCrCompRFID>
	outputs:		<LinkedList of PhdCrCompDigitalOutput's>
	led:			<PhdCrCompSimpleLED>

rfid:
	- a PhdCrCompRFID needed to get tag information of the RFID Phidget. See the PhdCrCompRFID class documentation for more information.

outputs:
	- a LinkedList. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrCompDigitalOutput's as the Phidget has. When the Phidget is detached the list is emptied. See the PhdCrCompDigitalOutput class documentation for more information.

led:
	- a PhdCrCompSimpleLED that is on-board on this type of Phidget. See the PhdCrCompSimpleLED class documentation for more information.


new callbacks:
	#TagFound (a tag was found by the rfid sensor),
	#TagLost (a tag was found by the rfid sensor),
	#OutputChange (an output has changed its value)


example indicating if a tag was found using the on-board LED
	rfid := PhdCrRFID new.
	rfid
		on: #Attach do: [ rfid rfidSensor setAntennaState: true ];
		on: #TagFound do: [ rfid led setState: true ];
		on: #TagLost do: [ rfid led setState: false ];
		openAny.
	"Now attach a RFID Phidget, hold a tag near the Phidget and then take it away."
	rfid close.!

!PhdCrRFID class methodsFor: 'callbacks' stamp: 'lk 5/30/2009 20:03'!
knownCallbacks
	"Returns a list of callback names (#symbols) the RFID Phidget knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #OutputChange #TagFound #TagLost )! !

!PhdCrRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:18'!
primCreate
	"Create a new the RFID object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primRFIDCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 20:31'!
deleteComponents
	"This method is called when the RFID Phidget is detached. It delets previously in 'initializeComponents' built components: outputs"

	outputs := LinkedList new.! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:11'!
initialize
	"additionally reset my components."

	super initialize.
	
	rfid := PhdCrCompRFID newWith: self.
	led := PhdCrCompSimpleLED newWith: self.
	outputs := LinkedList new.! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 21:56'!
initializeComponents
	"This method is called when a RFID Phidget is attached. It initializes its components:  outputs, rfid, led.
	The components will be deleted in 'deleteComponents' on detaching."

	outputs := PhdCrCompDigitalOutput getListFor: self.
	
	super initializeComponents.! !

!PhdCrRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:21'!
led
	^ led! !

!PhdCrRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:15'!
outputs
	^ outputs! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 17:31'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for #OutputChange, #TagFound and #TagLost."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #OutputChange).
	PhdCrCompDigitalOutput
		primSetSemaphoreIndexChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #TagFound).
	PhdCrCompRFID
		primSetSemaphoreIndexTagFound: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #TagLost).
	PhdCrCompRFID
		primSetSemaphoreIndexTagLost: semaIndex
		for: id.! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:18'!
resetState
	"when closing/attaching, turn led and all outputs off."

	super resetState.	
	self led isOn: false.
	self outputs do: [ :output | output value: false. ].! !

!PhdCrRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:20'!
rfidSensor
	^ rfid! !

PhdCrPhidget subclass: #PhdCrServo
	instanceVariableNames: 'motors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrServo commentStamp: 'sp 7/1/2009 12:35' prior: 0!
I am a representation of a Phidget called 'Servo' (here: 'Servo board') in Squeak.
To use the Servo board, create a new instance and open a connection to the hardware board with:
	servo := PhdCrServo new.
	servo openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured or the position of a motor was changed. For example:
	servo
		on: #Attach do: [ Transcript show:'servo was attached!!';cr ];
		on: #Detach do: [ Transcript show:'servo was detached!!';cr ];
		on: #PositionChange do: [ Transcript show:'A motor changed it's position!!' ].


Instance Variables
	motors:		<LinkedList of PhdCrCompServoMotor's>

motors:
	- a LinkedList of PhdCrCompServoMotor's. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrCompServoMotor's as the Phidget has. When the Phidget is detached the list is emptied. See the PhdCrCompServoMotor class documentation for more information.


new callbacks:
	#PositionChange (a motor has changed its position)
!

PhdCrServo subclass: #PhdCrAdvancedServo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrAdvancedServo commentStamp: 'sp 7/1/2009 13:07' prior: 0!
I am a representation of a Phidget called 'Advanced Servo' (here: 'Advanced Servo board') in Squeak.
Furthermore I'm a special Servo board. I additionally support acceleration and velocity for a motor.
To use the Advanced Servo board, create a new instance and open a connection to the hardware board with:
	advServo := PhdCrAdvancedServo new.
	advServo openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured, the position of a motor was changed, the velocity of a motor was changed or the current draw of a motor was changed. For example:
	servo
		on: #Attach do: [ Transcript show:'servo was attached!!';cr ];
		on: #Detach do: [ Transcript show:'servo was detached!!';cr ];
		on: #VelocityChange do: [ Transcript show:'A motor changed it's velocity!!' ];
		on: #CurrentChange do: [ Transcript show:'The current draw for a motor has changed!!' ].


Instance Variables
	motors:		<LinkedList of PhdCrCompAdvancedServoMotor's>

motors:
	- a LinkedList of PhdCrCompAdvancedServoMotor's. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrCompAdvancedServoMotor's as the Phidget has. When the Phidget is detached the list is emptied. See the PhdCrCompAdvancedServoMotor class documentation for more information.


new callbacks:
	#VelocityChange (a motor has changed its velocity),
	#CurrentChange (a motor has changed its current draw)
!

!PhdCrAdvancedServo class methodsFor: 'callbacks' stamp: 'sp 6/26/2009 06:18'!
knownCallbacks
	"Returns a list of callback names (#symbols) the AdvancedServo knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #VelocityChange #CurrentChange )! !

!PhdCrAdvancedServo class methodsFor: 'primitives' stamp: 'sp 6/26/2009 06:19'!
primCreate
	"Create a new the Advanced Servo object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primAdvancedServoCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrAdvancedServo methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:00'!
initializeComponents
	"This method is called when the Phidget is attached. It initializes its components:  motors.
	The components will be reset in 'deleteComponents' on detaching."

	motors := PhdCrCompAdvancedServoMotor getListFor: self.
	
	self resetState.! !

!PhdCrAdvancedServo methodsFor: 'initialize-open-close' stamp: 'sp 6/26/2009 06:16'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding a semaphore for #VelocityChange and #CurrentChange."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #VelocityChange).
	PhdCrCompAdvancedServoMotor
		primSetSemaphoreIndexVelocityChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #CurrentChange).
	PhdCrCompAdvancedServoMotor
		primSetSemaphoreIndexCurrentChange: semaIndex
		for: id.! !

!PhdCrAdvancedServo methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:18'!
resetState
	"when closing/attaching, reset acceleration."

	super resetState.	
	self motors do: [ :motor | motor acceleration: 0.0. ].! !

!PhdCrServo class methodsFor: 'callbacks' stamp: 'lk 5/12/2009 12:23'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Servo knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #PositionChange )! !

!PhdCrServo class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:18'!
primCreate
	"Create a new the Servo object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primServoCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 21:40'!
deleteComponents
	"This method is called when the Phidget is detached. It resets previously in 'initializeComponents' built components: motors"

	motors := LinkedList new.! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:12'!
initialize
	"additionally reset my components."

	super initialize.
	
	motors := LinkedList new.! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 21:56'!
initializeComponents
	"This method is called when the Phidget is attached. It initializes its components:  motots.
	The components will be reset in 'deleteComponents' on detaching."

	motors := PhdCrCompServoMotor getListFor: self.
	
	super initializeComponents.! !

!PhdCrServo methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:37'!
motors
	^ motors! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 22:16'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding a semaphore for #PositionChange."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #PositionChange).
	PhdCrCompServoMotor
		primSetSemaphoreIndexPositionChange: semaIndex
		for: id.! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:18'!
resetState
	"when closing/attaching, turn all motors off."

	super resetState.	
	self motors do: [ :motor | motor isEngaged: false. ].! !

PhdCrPhidget subclass: #PhdCrTextLCD
	instanceVariableNames: 'textLCD'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrTextLCD commentStamp: 'lk 7/9/2009 14:14' prior: 0!
I am a representation of a Phidget called 'Text LCD' in Squeak.
To use the Text LCD, create a new instance and open a connection to the hardware board with:
	lcd := PhdCrTextLCD new.
	lcd openAny.
A block can be set to be evaluated every time the hardware board was attached, detached or an error occoured. For example:
	lcd
		on: #Attach do: [ Transcript show:'lcd was attached!!';cr ];
		on: #Detach do: [ Transcript show:'lcd was detached!!';cr ];


Instance Variables
	textLCD:		<PhdCrCompTextLCD>

textLCD
	- a Text LCD Component that implements the "real" funcitionality of this Phidget. See its class comment for further information. When the Phidget is attached the rows of the component are initialized, when it is not, they are cleared.
!

!PhdCrTextLCD class methodsFor: 'callbacks' stamp: 'pt 7/14/2009 16:17'!
knownCallbacks
	"Returns a list of callback names (#symbols) the TextLCD knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #TextChange )! !

!PhdCrTextLCD class methodsFor: 'primitives' stamp: 'fs 7/5/2009 21:02'!
primCreate
	"Create a new the Text LCD object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primTextLCDCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrTextLCD methodsFor: 'initialize-open-close' stamp: 'lk 7/8/2009 18:42'!
deleteComponents
	"This method is called when a Phidget is detached.
	Delete the previously in 'initializeComponents' built component: textLCD.
	It must have empty rows now."
	
	textLCD initializeRowsEmpty.! !

!PhdCrTextLCD methodsFor: 'initialize-open-close' stamp: 'fs 7/5/2009 18:44'!
initialize
	"additionally reset my components."

	super initialize.
	
	textLCD := PhdCrCompTextLCD newWith: self.! !

!PhdCrTextLCD methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 21:57'!
initializeComponents
	"This method is called when a Phidget is attached.
	Initialise my component: textLCD.
	It will be 'uninitialised' in deleteComponents on detaching."
	
	textLCD initializeRows.
	
	super initializeComponents.! !

!PhdCrTextLCD methodsFor: 'initialize-open-close' stamp: 'lk 8/6/2009 22:24'!
resetState
	"when closing/attaching, reset texts and scrolling states."

	super resetState.
	1 to: self textLCD rows do: [ :rowNumber |
		self textLCD displayString: '' inRow: rowNumber. ].
	(self textLCD)
		stopScrolling;
		contrast: 150;
		setBacklightOn: true;
		setCursorOn: false;
		setCursorBlinking: false.! !

!PhdCrTextLCD methodsFor: 'callbacks' stamp: 'pt 7/14/2009 16:41'!
textChanged
	"Signals my textChange semaphore so that the #TextChange event workes."
	
	(semaphores at: #TextChange) signal.! !

!PhdCrTextLCD methodsFor: 'accessing' stamp: 'fs 7/5/2009 18:51'!
textLCD
	^ textLCD! !

!PhdCrTextLCD methodsFor: 'initialize-open-close' stamp: 'pt 7/14/2009 16:36'!
unregisterSemaphores
	"Unregister the semaphores which were previously registered with 'self registerSemaphores'.
	 Ignore my #TextChange event (because it is not an external object)"

	| mySema |
	self opened: false.
	mySema := semaphores at: #TextChange.
	mySema signal.
	semaphores removeKey: #TextChange.
	super unregisterSemaphores.
	semaphores at: #TextChange put: mySema.! !

BorderedMorph subclass: #PhdCrPhidgetManager
	instanceVariableNames: 'manager refreshButton attachedList attachedLabel openedList openedLabel label'
	classVariableNames: 'Semaphores OpenedPhidgets SemaphoreIndices'
	poolDictionaries: ''
	category: 'Phidget-Core'!

PhdCrPhidgetManager class
	instanceVariableNames: 'SampleImageForm'!

!PhdCrPhidgetManager commentStamp: 'lk 8/5/2009 22:17' prior: 0!
A PhdCrPhidgetManager is xxxxxxxxx.

Instance Variables

Class Variables
	OpenedPhidgets 		<Collection> (PhdCrPhidget)


OpenedPhidgets
	This is a list of all currently opened Phidgets.
	When a Phidget is opened it is added to this list. When it is closed it is removed from the list. This may also happen if the Phidget has become invalid (see checkValid).
	The getters are (on class side):
		allOpenedPhidgets - returns the list of all currently opened Phidgets.
		myOpenedPhidgets - returns a list of currently opend Phidgets of the receiver's type and its subclasses.
		myNonSpecialisedOpenedPhidgets - returns a list of currently opend Phidgets of the receiver's type (only).!

PhdCrPhidgetManager class
	instanceVariableNames: 'SampleImageForm'!

!PhdCrPhidgetManager class methodsFor: 'opened Phidgets list' stamp: 'lk 8/11/2009 18:25'!
addToOpenedPhidgets: aPhidget

	OpenedPhidgets := self allOpenedPhidgets copyWith: aPhidget.
	self openedPhidgetsListChanged.! !

!PhdCrPhidgetManager class methodsFor: 'opened Phidgets list' stamp: 'lk 8/5/2009 22:15'!
allOpenedPhidgets

	OpenedPhidgets ifNil: [ OpenedPhidgets := #() ].
	"list must be copied before checking elements valid, because if an element is invalid, it will be removed from the list"
	^ OpenedPhidgets copy select: [ :phidget | phidget checkValid ]! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/11/2009 20:54'!
attachedDevicesProcess: callbackName event: eventSelector
	"This method will be forked as an extra process for every callback that a phidget manager needs.
	The semaphores were previously initialised and registered as external objects.
	self class knownCallbacks lists all possible semaphore names."
	
	(self semaphores at: callbackName) wait.
	
	[true] whileTrue: [
		self triggerEvent: eventSelector.
		(self semaphores at: callbackName) wait
	].! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/11/2009 20:52'!
checkSemaphores
	"check that the semaphores are registered in the plugin."
	
	| externalObjects included |
	externalObjects := Smalltalk specialObjectsArray at: 39.
	
	included := Semaphores
		ifNil: [ false ]
		ifNotNil: [
			externalObjects includesAllOf: {
				Semaphores at: #Attach.
				Semaphores at: #Detach.
				Semaphores at: #Error.
			}.
		].
	
	included ifFalse: [ self resetCallbacks ].
	
	self primSetSemaphoreIndexAttach: (self semaphoreIndices at: #Attach).
	self primSetSemaphoreIndexDetach: (self semaphoreIndices at: #Detach).
	self primSetSemaphoreIndexError: (self semaphoreIndices at: #Error).! !

!PhdCrPhidgetManager class methodsFor: 'PartsBin' stamp: 'lk 8/12/2009 12:14'!
clearSampleImages
	"clear all sample images that will get thumbnails in ObjectsTool."

	self withAllSubclassesDo: [ :aClass | aClass sampleImageForm: nil ].! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/11/2009 20:55'!
createAttachedDevicesProcessBlock: callbackName
	"Create a callback process block to be forked.
	Must be swapped out to work in loops."

	^ [ self
			attachedDevicesProcess: callbackName
			event: (callbackName , #Event) asSymbol ]! !

!PhdCrPhidgetManager class methodsFor: 'PartsBin' stamp: 'lk 8/12/2009 11:23'!
descriptionForPartsBin
	"also set an Image Form, so no instance is created when creating a Thumbnail"
	
	^ self descriptionForPartsBinBasic
		sampleImageForm: self getSampleImageForm;
		yourself.! !

!PhdCrPhidgetManager class methodsFor: 'PartsBin' stamp: 'lk 8/12/2009 11:23'!
descriptionForPartsBinBasic

	^ self
		partName:		'PhidgetManager'
		categories:		#('Phidgets')
		documentation:  'A Manager View for Phidget Objects'! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/11/2009 20:55'!
forkAttachedDevicesProcesses

	self knownCallbacks do: [ :callbackName |
		(self createAttachedDevicesProcessBlock: callbackName) fork ].! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list' stamp: 'lk 8/6/2009 11:03'!
getAttachedDeviceClassIdAt: index
	"Gets the device class id of the attached device in the list at given index. If there was an error, -1 is returned."
	
	^ self primGetAttachedDeviceClass: index - 1.! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list' stamp: 'lk 8/6/2009 11:02'!
getAttachedDeviceLabelAt: index
	"Gets the label of the attached device in the list at given index. If there was an error, an empty string is returned."
	
	| result |
	result := self primGetAttachedDeviceLabel: index - 1.
	^ result
		ifNil: [ '' ]
		ifNotNil: [ result asString ]! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list' stamp: 'lk 8/6/2009 10:10'!
getAttachedDeviceNameAt: index
	"Gets the name of the attached device in the list at given index. If there was an error, an empty string is returned."
	
	| result |
	result := self primGetAttachedDeviceName: index - 1.
	^ result
		ifNil: [ '' ]
		ifNotNil: [ result asString ]! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list' stamp: 'lk 8/6/2009 10:41'!
getAttachedDeviceSerialNumberAt: index
	"Gets the serial number of the attached device in the list at given index. If there was an error, -1 is returned."
	
	^ self primGetAttachedDeviceSerialNumber: index - 1.! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list' stamp: 'lk 8/6/2009 11:03'!
getAttachedDeviceTypeIdAt: index
	"Gets the device type id of the attached device in the list at given index. If there was an error, -1 is returned."
	
	^ self primGetAttachedDeviceType: index - 1.! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list' stamp: 'lk 8/7/2009 15:19'!
getAttachedDevicesInfos

	| count list |
	count := self getNumberOfAttachedDevices.
	
	"create and initialise list"
	list := Array new: count.
	1 to: count do: [ :i | | entry |
		entry := PhdCrAttachedDeviceInfo new
			deviceName: (self getAttachedDeviceNameAt: i);
			serialNumber: (self getAttachedDeviceSerialNumberAt: i);
			label: (self getAttachedDeviceLabelAt: i);
			deviceClassId: (self getAttachedDeviceClassIdAt: i);
			deviceTypeId: (self getAttachedDeviceTypeIdAt: i).
		list at: i put: entry ].
	
	^ list
	! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list' stamp: 'lk 8/7/2009 15:19'!
getNumberOfAttachedDevices
	"answer the size of the internal list of attached phidgets, or 0 if an error occurred."

	| result |
	result := self primGetNumberOfAttachedDevices.
	^ result == -1
		ifTrue: [ 0 ]
		ifFalse: [ result ]! !

!PhdCrPhidgetManager class methodsFor: 'PartsBin' stamp: 'lk 8/12/2009 12:14'!
getSampleImageForm
	"Get a Image Form that will be transofmed into a Thumbnail for PartsBin.
	Do not create a real instance of myself"
	"Cache the image because this method is called quite often when dealing with the ObjectsTool."
	
	self sampleImageForm ifNil: [
		self sampleImageForm: (self newThumbnailStandAlone imageFormDepth: 32) ].
	^ self sampleImageForm! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/7/2009 18:02'!
knownCallbacks
	"Returns a list of callback names (#symbols) the PhidgetManager knows."
	
	^ #( #Attach #Detach #Error )! !

!PhdCrPhidgetManager class methodsFor: 'error codes' stamp: 'lk 8/5/2009 22:13'!
lastError
	"Returns the error code of the last error in the PhidgetPlugin, or -1 if something went wrong.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h in line 2250 ff."

	^ self primLastError! !

!PhdCrPhidgetManager class methodsFor: 'opened Phidgets list' stamp: 'lk 8/5/2009 22:15'!
myNonSpecialisedOpenedPhidgets
	"answer a collection with all currently opened Phidgets of (exactly) my type."

	^ self allOpenedPhidgets select: [ :phidget |
		phidget class == self ]! !

!PhdCrPhidgetManager class methodsFor: 'opened Phidgets list' stamp: 'lk 8/5/2009 22:15'!
myOpenedPhidgets
	"answer a collection with all currently opened Phidgets of my type and subclasses of me."

	^ self allOpenedPhidgets select: [ :phidget |
		phidget class == self or: [ phidget class inheritsFrom: self ] ]! !

!PhdCrPhidgetManager class methodsFor: 'PartsBin' stamp: 'lk 8/12/2009 11:24'!
newThumbnailStandAlone
	"Create an instance of myself that does not 'work', but can be created a Thumbnail for PartsBin from."

	^ self basicNew initializeForThumbnail! !

!PhdCrPhidgetManager class methodsFor: 'opened Phidgets list' stamp: 'lk 8/11/2009 18:25'!
openedPhidgetsListChanged

	self triggerEvent: #openedPhidgetsListChangedEvent.! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/6/2009 11:05'!
primGetAttachedDeviceClass: index
	"Answer the device class id of the attached device in list at position index. Answers -1 if an error occurred."

	<primitive: 'primGetAttachedDeviceClass' module: 'PhidgetPlugin'>
	^ -1! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/6/2009 11:05'!
primGetAttachedDeviceLabel: index
	"Answer a byte array of the label of the attached device in list at position index. Answers nil if an error occurred."

	<primitive: 'primGetAttachedDeviceLabel' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/6/2009 11:05'!
primGetAttachedDeviceName: index
	"Answer a byte array of the name of the attached device in list at position index. Answers nil if an error occurred."

	<primitive: 'primGetAttachedDeviceName' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/6/2009 10:42'!
primGetAttachedDeviceSerialNumber: index
	"Answer the serial number of the attached device in list at position index. Answers -1 if an error occurred."

	<primitive: 'primGetAttachedDeviceSerialNumber' module: 'PhidgetPlugin'>
	^ -1! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/6/2009 11:06'!
primGetAttachedDeviceType: index
	"Answer the device type id of the attached device in list at position index. Answers -1 if an error occurred."

	<primitive: 'primGetAttachedDeviceType' module: 'PhidgetPlugin'>
	^ -1! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/7/2009 15:20'!
primGetNumberOfAttachedDevices
	"Return the number of attached (but not necissarily opened) Phidgets.
	If an error occurred, -1 is returned."

	<primitive: 'primGetNumberOfAttachedDevices' module: 'PhidgetPlugin'>
	^ -1! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/5/2009 22:14'!
primLastError
	"Returns the error code of the last error in the Plugin, or -1 if something went wrong.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h in line 2250 ff."

	<primitive: 'primLastError' module: 'PhidgetPlugin'>
	^ -1! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/7/2009 18:41'!
primSetSemaphoreIndexAttach: semaphoreIndex
	"registers the given Semaphore to be signaled when a phidget is attached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetPMSemaphoreIndexAttach' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/7/2009 18:41'!
primSetSemaphoreIndexDetach: semaphoreIndex
	"registers the given Semaphore to be signaled when a phidget is detached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetPMSemaphoreIndexDetach' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidgetManager class methodsFor: 'primitives' stamp: 'lk 8/7/2009 18:41'!
primSetSemaphoreIndexError: semaphoreIndex
	"registers the given Semaphore to be signaled when an error occurres.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetPMSemaphoreIndexError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidgetManager class methodsFor: 'opened Phidgets list' stamp: 'lk 8/11/2009 18:25'!
removeFromOpenedPhidgets: aPhidget

	OpenedPhidgets := self allOpenedPhidgets copyWithout: aPhidget.
	self openedPhidgetsListChanged.! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/11/2009 20:54'!
resetCallbacks

	self knownCallbacks do: [ :callbackName |
		Semaphores ifNotNil: [
			Smalltalk unregisterExternalObject: (Semaphores at: callbackName).
			(Semaphores at: callbackName) copy do: [ :process |
				process terminate ]
		].
	].
	SemaphoreIndices := nil.
	Semaphores := nil.
	
	! !

!PhdCrPhidgetManager class methodsFor: 'PartsBin' stamp: 'lk 8/12/2009 12:14'!
sampleImageForm

	^ SampleImageForm! !

!PhdCrPhidgetManager class methodsFor: 'PartsBin' stamp: 'lk 8/12/2009 12:14'!
sampleImageForm: aForm

	SampleImageForm := aForm.! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/7/2009 18:09'!
semaphoreIndices

	SemaphoreIndices ifNil: [
		SemaphoreIndices := Dictionary new.
		self knownCallbacks do: [ :callbackName | | index |
			index := Smalltalk registerExternalObject: (self semaphores at: callbackName).
			SemaphoreIndices add: callbackName -> index ].
		].
	
	^ SemaphoreIndices! !

!PhdCrPhidgetManager class methodsFor: 'attached Phidgets list - callbacks' stamp: 'lk 8/11/2009 20:48'!
semaphores

	Semaphores ifNil: [
		Semaphores := Dictionary new.
		
		"Create the semaphores and fork processes waiting for them. The processes will trigger appropriate events."
		self knownCallbacks do: [ :callbackName |
			Semaphores add: callbackName -> Semaphore new ].
		self forkAttachedDevicesProcesses.
	].
	
	^ Semaphores! !

!PhdCrPhidgetManager methodsFor: 'initialize-release' stamp: 'lk 8/12/2009 11:08'!
basicInitialize

	super initialize.
	
	self
		width: 350;
		color: Color white;
		borderWidth: 1;
		useRoundedCorners;
		createAndAddSubMorphs;
		height: self minHeight.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
createAndAddAttachedLabel

	attachedLabel := LabelMorph contents: 'Attached Phidgets:'.
	self addMorph: attachedLabel.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
createAndAddAttachedList

	attachedList := MultiColumnListMorph
		on: nil
		list: #()
		initialSelectedObject: nil
		changeSelected: nil.
	attachedList
		header: #( 'Name' 'SN' 'Label' 'Class Code' 'Type Code' );
		height: 150.
	
	self addMorph: attachedList.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
createAndAddLabel

	label := LabelMorph
		contents: 'Phidget Manager'
		font: nil
		emphasis: 1.		"bold"
	
	self addMorph: label.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
createAndAddOpenedLabel

	openedLabel := LabelMorph contents: 'Opened Phidgets:'.
	self addMorph: openedLabel.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
createAndAddOpenedList

	openedList := MultiColumnListMorph
		on: nil
		list: #()
		initialSelectedObject: nil
		changeSelected: nil.
	openedList header: #( 'Name' ).
	
	self addMorph: openedList.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:30'!
createAndAddRefreshButton

	refreshButton := (SimpleButtonMorph newWithLabel: 'Refresh')
		color: (Color r: 0.494 g: 0.694 b: 0.854);
		borderWidth: 1;
		borderColor: Color gray;
		target: self;
		actionSelector: #refresh;
		setBalloonText: 'Refresh the lists if the image was shut down and reloaded';
		useRoundedCorners;
		yourself.
	
	self addMorph: refreshButton.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
createAndAddSubMorphs

	self
		createAndAddLabel;
		createAndAddRefreshButton;
		createAndAddAttachedLabel;
		createAndAddAttachedList;
		createAndAddOpenedLabel;
		createAndAddOpenedList;
		
		repositionAll.! !

!PhdCrPhidgetManager methodsFor: 'initialize-release' stamp: 'lk 8/11/2009 21:13'!
delete

	PhdCrPhidgetManager removeActionsWithReceiver: self.
	
	super delete.! !

!PhdCrPhidgetManager methodsFor: 'updating view' stamp: 'lk 8/11/2009 21:08'!
extent: aPoint

	| newExtent newWidth newHeight |
	
	newWidth := aPoint x max: self minWidth.
	newHeight := aPoint y max: self minHeight.
	newExtent := newWidth @ newHeight.
	super extent: newExtent.
	
	self repositionAll.! !

!PhdCrPhidgetManager methodsFor: 'initialize-release' stamp: 'lk 8/12/2009 11:08'!
initialize
	
	self
		basicInitialize;
		registerEvents;
		refresh.! !

!PhdCrPhidgetManager methodsFor: 'initialize-release' stamp: 'lk 8/12/2009 11:24'!
initializeForThumbnail
	"initialize an instance of myself that does not fully 'work', but can be created a Thumbnail for PartsBin from.
	-> Do not register any events"

	self basicInitialize.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
paddingLeftRight
	^ 10! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
paddingTopBottom
	^ 10! !

!PhdCrPhidgetManager methodsFor: 'updating view' stamp: 'lk 8/11/2009 21:13'!
refresh

	PhdCrPhidgetManager checkSemaphores.
	self
		refreshAttached;
		refreshOpened.! !

!PhdCrPhidgetManager methodsFor: 'updating view' stamp: 'lk 8/11/2009 21:08'!
refreshAttached

	| attachedDevicesInfos attachedDevicesArray |
	
	attachedList ifNil: [ ^ self ].
	
	"attachedDevicesInfos := manager attachedDevices."
	attachedDevicesInfos := PhdCrPhidgetManager getAttachedDevicesInfos.
	attachedDevicesArray := Array new: attachedDevicesInfos size.
	
	1 to: attachedDevicesInfos size do: [ :i | | info |
		info := attachedDevicesInfos at: i.
		attachedDevicesArray at: i put: info asStringArray.
	].

	attachedList list: attachedDevicesArray.! !

!PhdCrPhidgetManager methodsFor: 'updating view' stamp: 'lk 8/11/2009 21:08'!
refreshOpened

	| openedPhidgets openedArray |
	
	openedPhidgets := PhdCrPhidgetManager allOpenedPhidgets.
	openedArray := Array new: openedPhidgets size.

	1 to: openedPhidgets size do: [ :i | | phidget |
		phidget := openedPhidgets at: i.
		openedArray at: i put: phidget asString.
	].

	openedList list: openedArray.! !

!PhdCrPhidgetManager methodsFor: 'initialize-release' stamp: 'lk 8/11/2009 21:27'!
registerEvents

	"register events for attached list"
	PhdCrPhidgetManager
		when: #AttachEvent
		send: #refreshAttached
		to: self.
	PhdCrPhidgetManager
		when: #DetachEvent
		send: #refreshAttached
		to: self.
	
	"register events for opened list"
	PhdCrPhidgetManager
		when: #openedPhidgetsListChangedEvent
		send: #refreshOpened
		to: self.! !

!PhdCrPhidgetManager methodsFor: 'updating view' stamp: 'lk 8/11/2009 21:08'!
repositionAll

	self
		repositionRefreshButton;
		repositionLabel;
		repositionAttachedLabel;
		repositionAttachedList;
		repositionOpenedLabel;
		repositionOpenedList;
		
		updateMinBounds.! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
repositionAttachedLabel

	| maxY |
	
	(attachedLabel notNil and: [label notNil] and: [refreshButton notNil]) ifTrue: [
		maxY := label bottom max: refreshButton bottom.
		attachedLabel position: (self left @ maxY) + (self paddingLeftRight @ self paddingTopBottom)
	].! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:09'!
repositionAttachedList
	
	(attachedList notNil and: [attachedLabel notNil]) ifTrue: [
		attachedList
			width: self width - ( 2* self paddingLeftRight);
			position: attachedLabel bottomLeft + ( 0 @ self paddingTopBottom ).
	].! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
repositionLabel

	label ifNotNil: [
		label position: self topLeft + (self paddingLeftRight @ self paddingTopBottom).
	].! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
repositionOpenedLabel

	(openedLabel notNil and: [attachedList notNil]) ifTrue: [
		openedLabel position:
			attachedList bottomLeft +
			(0 @ (self paddingTopBottom * 1.5) rounded).
	].! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
repositionOpenedList

	(openedList notNil and: [openedLabel notNil]) ifTrue: [
		openedList
			width: self width - ( 2* self paddingLeftRight);
			position: openedLabel bottomLeft + ( 0 @ self paddingTopBottom ).
	].! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
repositionRefreshButton

	refreshButton ifNotNil: [
		refreshButton position:
			self topRight -
			( (refreshButton width + self paddingLeftRight) @ (self paddingTopBottom negated) ).
	].! !

!PhdCrPhidgetManager methodsFor: 'private' stamp: 'lk 8/11/2009 21:08'!
updateMinBounds

	({ label. refreshButton. attachedLabel. openedList. openedLabel } allSatisfy: [ :item | item notNil ]) ifTrue: [
		self minHeight: 2 * self paddingTopBottom + ( openedList bottom - label top ).
		self minWidth:
			2 * self paddingLeftRight +
			{ label width + self paddingLeftRight + refreshButton width.
			  attachedLabel width.
			  openedLabel  width } max
	].! !
